#ifndef ARMOR_DETECT_H
#define ARMOR_DETECT_H

#include<ros/ros.h>
#include<iostream>
#include<vector>
#include<algorithm>
#include<cv_bridge/cv_bridge.h>
#include<sensor_msgs/image_encodings.h>
#include<opencv2/core/core.hpp>
#include<opencv2/highgui/highgui.hpp>
#include<opencv2/imgproc/imgproc.hpp>
#include<image_transport/image_transport.h>
#include"armor_detect/armor_center.h"
#include"armor_detect/armor_msg.h"
using namespace cv;
using namespace std;

class ARMOR_DETECT
{
private:
    int width,height;

    int thresh;
    double fx;
    double fy;
    double cx;
    double cy;
    double function_a;
    double function_b;
    bool show_img;
    bool blueArmor;
    double g_t0;
    double dist_c[5] = {0.064209, -0.127583, 0.000267, -0.007102, 0.000000};
    double cam[9] = {509.473611, 0.000000,   296.059930,
              0.000000,   507.459976, 238.831330,
              0.000000,   0.000000,   1.000000};

    vector<Point2f> armor_points;
    vector<Point3f> obj_p;
    vector<Point2f> img_p;

    cv::Mat camera_matrix;
    cv::Mat dist_coeffs;
    cv::Mat obj_points;
    cv::Mat img_points;

    cv::Mat rvec;
    cv::Mat tvec;

    armor_detect::armor_msg armor_info;
    armor_detect::armor_center armor;
    ros::Publisher tagcenter_pub_;
    image_transport::ImageTransport it_;
    image_transport::Subscriber image_sub_;
    image_transport::Publisher image_pub_;



public:
    ARMOR_DETECT(ros::NodeHandle& nh_,ros::NodeHandle& pnh);
    ~ARMOR_DETECT();
    double tic();
    void convert_callback(const sensor_msgs::ImageConstPtr& msg);
    void find_armor(Mat &img);
    void image_show(Mat &img);
    float white_point_scale (Mat &src,Point left,Point right);
    void cal_armor_info(std::vector<cv::Point2f> &armor_points, cv::RotatedRect left_light, cv::RotatedRect right_light);

};

#endif // ARMOR_DETECT_H

