#include <iostream>
#include <string>
#include <list>
#include <vector>
#include <map>
#include <stack>

#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>
using namespace std;
using namespace cv;


#include "armor_detect/armor_detect.h"
static const string INPUT = "Input";
static const string OUTPUT = "Output";

ARMOR_DETECT::ARMOR_DETECT(ros::NodeHandle& nh_,ros::NodeHandle& pnh):it_(nh_)
{
    if(!pnh.getParam("function_a", function_a)){
     ROS_WARN("No function_a tags specified");
   }
     image_sub_ = it_.subscribe("/image_raw",
                               1, &ARMOR_DETECT::convert_callback, this);
     //image_pub_ = it_.advertise("/image_converter/output_video2", 1);

     tagcenter_pub_ = nh_.advertise<armor_detect::armor_msg>("armor_info",1000);
//1
    pub_goal = nh_.advertise<move_base_msgs::MoveBaseGoal>("camera_goal", 10);
    //Set the goal value to geometry_msgs::PoseStamped.
    goal_pose.target_pose.header.stamp    = ros::Time::now();
    goal_pose.target_pose.header.frame_id = "base_link";
    goal_pose.target_pose.pose.position.x = 0;
    goal_pose.target_pose.pose.position.y = 0;
    goal_pose.target_pose.pose.position.z = 0;

    goal_pose.target_pose.pose.orientation.w = 0;
    goal_pose.target_pose.pose.orientation.x = 0;
    goal_pose.target_pose.pose.orientation.y = 0;
//1

     pnh.param<int>("width", width, 640);
     pnh.param<int>("height", height, 480);

     pnh.param<int>("thresh", thresh, 35);

     pnh.param<bool>("show_img", show_img, false);
     pnh.param<bool>("blueArmor", blueArmor, true);

     pnh.param<double>("function_a", function_a, 1594.0);
     pnh.param<double>("function_b", function_b, -0.9427);

     pnh.param<double>("fx", fx, 489.355407);
     pnh.param<double>("fy", fy, 490.524912);
     pnh.param<double>("cx", cx, 316.109536);
     pnh.param<double>("cy", cy, 247.635580);

     //pnh.getParam("iLowV", function_a);
     //pnh.getParam("iHighV", function_b);


     armor.x = 0.0;
     armor.y = 0.0;
     armor.d = 0.0;
     armor.find_armio = false;

     armor_info.x = 0.0;
     armor_info.y = 0.0;
     armor_info.d = 0.0;
     armor_info.detected = false;
         //init pnp param
    obj_p.push_back(cv::Point3f(0.0,   0.0,  0.0));
    obj_p.push_back(cv::Point3f(140,   0.0,  0.0));
    obj_p.push_back(cv::Point3f(140,   60,   0.0));
    obj_p.push_back(cv::Point3f(0.0,   60,   0.0));

    camera_matrix = cv::Mat(3, 3, CV_64F, cam);
    dist_coeffs   = cv::Mat(5, 1, CV_64F, dist_c);

    rvec = cv::Mat::ones(3,1,CV_64F);
    tvec = cv::Mat::ones(3,1,CV_64F);
    
    dis_list_error_num = 0;
    dis_list_size = 6;
//test
    g_t0 = 0.0;
}

ARMOR_DETECT::~ARMOR_DETECT()
{

}
double ARMOR_DETECT::tic() {
    struct timeval t;
    gettimeofday(&t, NULL);
    return ((double)t.tv_sec + ((double)t.tv_usec)/1000000.);
}

void ARMOR_DETECT::convert_callback(const sensor_msgs::ImageConstPtr& msg)
{
    cv_bridge::CvImagePtr cv_ptr;
     try
     {
         cv_ptr =  cv_bridge::toCvCopy(msg, sensor_msgs::image_encodings::BGR8);
     }
     catch(cv_bridge::Exception& e)
     {
         ROS_ERROR("cv_bridge exception: %s", e.what());
         return;
     }
     g_t0 = tic();

     find_armor(cv_ptr->image);
     armor_info.x = armor.x;
     armor_info.y = armor.y;
     armor_info.d = armor.d;
     armor_info.detected = armor.find_armio;
     tagcenter_pub_.publish(armor_info);
//2
	goal_pose.target_pose.header.frame_id = "base_link";
	goal_pose.target_pose.header.stamp = ros::Time::now();
	goal_pose.target_pose.pose.position.x = armor.d;
	goal_pose.target_pose.pose.position.y = 0;
	goal_pose.target_pose.pose.position.z = 1;
	//tf::Quaternion q;
	//q.setRPY(0, 0, pan_yaw);
	goal_pose.target_pose.pose.orientation.x= 0;
	goal_pose.target_pose.pose.orientation.y= 0;
	goal_pose.target_pose.pose.orientation.z= 0;
	goal_pose.target_pose.pose.orientation.w= 1;
	pub_goal.publish(goal_pose);
//2
     ROS_INFO (" over cost time = [%f]",tic()-g_t0);
     if(show_img)
     image_show(cv_ptr->image);

}

float ARMOR_DETECT::white_point_scale (Mat& src,Point left,Point right)
{
    if((left.x < 0) || (left.x > src.cols)||(left.y < 0) || (left.y > src.rows))
        return 0;
    if((right.x < 0) || (right.x > src.cols)||(right.y < 0) || (right.y > src.rows))
        return 0;

        Mat imgThresholded;
        Rect rect;
        rect.width = abs(left.x - right.x);

        rect.x = min(left.x ,right.x);
        rect.y = max(min(left.y ,right.y) - rect.width/3 , 0);
        rect.y =  min(rect.y , src.rows-1);
        rect.height = min(src.rows - rect.y ,rect.width/2);
        
        cvtColor(src(rect), imgThresholded, CV_BGR2GRAY );//灰度化
        threshold(imgThresholded,imgThresholded,80,255,THRESH_BINARY);
        //imshow("src(rect)",src(rect));
        float count = countNonZero(imgThresholded);


        return count/rect.area();
}

void ARMOR_DETECT::image_show(Mat &img)
{
    Mat img_out;
    cvtColor(img, img_out, CV_BGR2GRAY);  //转换成灰度图象
    resize(img_out,img_out,cv::Size(width,height));
    imshow(INPUT, img);
    
    //imshow(OUTPUT, img_out);
    moveWindow(INPUT, 0, 0);
    //moveWindow(OUTPUT, 700, 0);
    waitKey(1);
}

void ARMOR_DETECT::find_armor(Mat &src)
{

    vector<Mat> imgBGR;
    Mat temp = Mat::zeros( src.size(), CV_8UC1);
    Mat src_gray = Mat::zeros( src.size(), CV_8UC1);
    Mat canny_output = Mat::zeros( src.size(), CV_8UC1);
    Mat element = getStructuringElement(MORPH_ELLIPSE, Size(11, 11));
    Mat element_small = getStructuringElement(MORPH_ELLIPSE, Size(3, 3));
    split(src,imgBGR);
    if(blueArmor)
    {
        temp = imgBGR[0] - imgBGR[2]; //blue
    }
    else
    {
        temp = imgBGR[2] - imgBGR[0]; //red
    }

    threshold(temp,temp,thresh,255,THRESH_BINARY);

    morphologyEx(temp, temp, MORPH_CLOSE, element_small);

    inRange(src, cv::Scalar(180,180,180), cv::Scalar(255,255,255), src_gray);

    dilate(src_gray,src_gray,element_small);
    
    //if(show_img)
    //{

    //imshow("src_gray",src_gray);
    //moveWindow("src_gray",700,500);
    //}

    //查找轮廓
    vector<vector<Point> > contours;
    vector<Vec4i> hierarchy;
    findContours( src_gray, contours, hierarchy, RETR_EXTERNAL, CHAIN_APPROX_NONE);
    //计算轮廓矩
    vector<Moments> mu(contours.size() );
    for( int i = 0; i < contours.size(); i++ )
    {
        mu[i] = moments( contours[i], false );
    }
    //计算轮廓的质心
    vector<Point2f> mc( contours.size());

    for( int i = 0; i < contours.size(); i++ )
    {
        mc[i] = Point2d( mu[i].m10/mu[i].m00 , mu[i].m01/mu[i].m00 );
    }

    //画轮廓及其质心并显示
    Mat drawing = Mat::zeros( canny_output.size(), CV_8UC3 );
    float a,b,c;
    vector<float> angle( contours.size() );
    vector<int> contours_id;
    RNG rng(12345);
    for( int i = 0; i< contours.size(); i++ )
    {
        Scalar color = Scalar( rng.uniform(0, 255), rng.uniform(0,255), rng.uniform(0,255) );
        //drawContours( drawing, contours, i, color, 2, 8, hierarchy, 0, Point() );

        a = mu[i].m20/mu[i].m00 -  pow(mc[i].x,2);
        b = 2*( mu[i].m11/mu[i].m00 -  mc[i].x*mc[i].y);
        c = mu[i].m02/mu[i].m00 -  pow(mc[i].y,2);
        angle[i] = atan2(b,a-c)/2*180/3.1415926;
        
	if((angle[i] >= 90)||(angle[i] <= -90))
		angle[i] = 90;
        if(angle[i] >= 0)
        {
            angle[i] = angle[i] -90;
        }
        else
        {
            angle[i] = angle[i] + 90;
        }
        
        float length = arcLength(contours[i],true);

	//cout<<"length"<<i<<" = "<<length<<endl;
        //cout<<"mu [ "<<i<<" ] = "<<mu[i].m00<<endl;
        if((fabs(angle[i]) < 55)&&(length >5))
        {
            circle( drawing, mc[i], 4, color, -1, 8, 0 );
            contours_id.push_back(i);
        }

    }

    //if(show_img)
    //{

    ///imshow("drawing",drawing);
    //moveWindow("drawing",0,500);
    //}
    float x = 0.0;
    float y = 0.0;

    float hor_angle_limit = 0.37;
    float distance_limit = 0.35;
    float distance_limit_max = 1.5;
    float ver_angle_limit = 25;
    float length_similar_limit = 2.5;
    float scale_limit = 0.4;
    int left_id = 0;
    int right_id = 1;
    bool find_armio = false;
    //cout<<"contours_id.size() = "<<contours_id.size()<<endl;
    //cout<<"contours.size() = "<<contours.size()<<endl;
    for( int i = 0; i< contours_id.size(); i++ )
    {
        float angle_temp = angle[contours_id[i]];
         Point2f mc_temp = mc[contours_id[i]];

         double length_temp = arcLength(contours[contours_id[i]], true);

        for(int j = i+1; j< contours_id.size(); j++)
        {

//                        if(mc_temp == mc[contours_id[j]])
//                        {
//                            continue;
//                        }

//                        if(angle_temp * angle[contours_id[j]] < 0)
//                        {
//                            continue;
//                        }


            x = mc_temp.x - mc[contours_id[j]].x;
            y = mc_temp.y - mc[contours_id[j]].y;
            cout<<"fabs(atan(y/x)) = "<<fabs(atan(y/x))<<endl;
            if(fabs(atan(y/x)) > hor_angle_limit)
            {
                continue;
            }
            //hor_angle_limit = fabs(atan(y/x));





	    
            float scale = white_point_scale(src,mc[contours_id[i]],mc[contours_id[j]]);
		cout<<"scale = "<<scale<<endl;
            if(scale < scale_limit)
            {
                 continue;
            }
            //scale_limit = scale;



            double length_similar = arcLength(contours[contours_id[j]], true)/length_temp;
            cout<<"fabs(length_similar - 1) = "<<fabs(length_similar - 1)<<endl;
            if(fabs(length_similar - 1) > length_similar_limit)
            {
                continue;
            }
            //length_similar_limit = fabs(length_similar - 1);

            cout<<"fabs(angle_temp-angle[contours_id[j]]) = "<<fabs(angle_temp-angle[contours_id[j]])<<endl;
            if(  (fabs(angle_temp-angle[contours_id[j]]) > ver_angle_limit) )
            {
                continue;
            }
            //ver_angle_limit = fabs(angle_temp-angle[contours_id[j]]);

	    cout<<" length_temp / fabs(x) ="<<length_temp / fabs(x)<<endl;
            if(length_temp / fabs(x) < distance_limit)
            {
                continue;
            }

           if(length_temp / fabs(x) > distance_limit_max)
	   {
	      continue;
	   }
	   distance_limit_max = length_temp / fabs(x);
           //distance_limit = 

	cout<<"find"<<endl;


            //cout<<" length_temp / fabs(x) ="<<length_temp / fabs(x)<<endl;

            //distance_limit = length_temp / fabs(x);





            find_armio = true;
            left_id = i;
            right_id = j;
        }
    }

    //double pnpTime = tic();
    if(find_armio)
    {
	if(mc[contours_id[left_id]].x > mc[contours_id[right_id]].x)
	{
	int temp_id = left_id;
	left_id = right_id;
	right_id =temp_id;
	}





        Point2f armio_center ,up_armio_center,low_armio_center;

        armio_center.x = (mc[contours_id[left_id]].x + mc[contours_id[right_id]].x)/2;
        armio_center.y = (mc[contours_id[left_id]].y + mc[contours_id[right_id]].y)/2;

        float height = (arcLength(contours[contours_id[left_id]], true) + arcLength(contours[contours_id[right_id]], true))/4 ;
        float width = fabs(mc[contours_id[left_id]].x - mc[contours_id[right_id]].x);

        up_armio_center.x = armio_center.x;
        up_armio_center.y = armio_center.y - height/2;

        low_armio_center.x = armio_center.x;
        low_armio_center.y = armio_center.y + height/2;

        if(show_img)
        {
            Scalar color = Scalar( rng.uniform(0, 255), rng.uniform(0,255), rng.uniform(0,255) );
            circle( src, mc[contours_id[left_id]], 2, color, -1, 8, 0 );
            circle( src, mc[contours_id[right_id]], 2, color, -1, 8, 0 );
            circle( src, armio_center, 2, color, -1, 8, 0 );
            line(src, mc[contours_id[left_id]], mc[contours_id[right_id]], cvScalar(0,0,200), 2);
            line(src, low_armio_center, up_armio_center, cvScalar(0,0,200), 2);
            circle( src, Point(cx,cy), 4, cvScalar(0,0,255), -1, 8, 0 );
	    line(src, armio_center, Point(cx,cy), cvScalar(0,255,0), 1);
        }
//pnp
	RotatedRect minEllipseLeft,minEllipseRight;

	minEllipseLeft = fitEllipse(contours[contours_id[left_id]]);
        minEllipseRight = fitEllipse(contours[contours_id[right_id]]);

	cal_armor_info(armor_points,minEllipseLeft,minEllipseRight);
        for(int i=0;i < 3;i++)
	{
	cout<<"armor_points["<<i<<"]"<<armor_points[i]<<endl;
	line(src, armor_points[i], armor_points[i+1], cvScalar(0,255,0), 1);
	}

        solvePnP(obj_p, armor_points, camera_matrix, dist_coeffs, rvec, tvec);

	armor_points.clear();
                double enemy_dis  = tvec.at<double>(2)/1000.0;//Z distance
                double enemy_disy = tvec.at<double>(1)/1000.0;//Y distance
                double enemy_disx = tvec.at<double>(0)/1000.0;//X distance
                double enemy_angz = rvec.at<double>(2);//Rotate around z axis
                double enemy_ang = rvec.at<double>(1);//Rotate around y axis
                double enemy_angx = rvec.at<double>(0);//Rotate around x axis
                std::cout<<"Enemy distancez:"<<enemy_dis<<std::endl;
                std::cout<<"Enemy distancey:"<<enemy_disy<<std::endl;
                std::cout<<"Enemy distancex:"<<enemy_disx<<std::endl;
	double sum = 0.0;
	double mean = 0.0;
	if(dis_list.size()<dis_list_size)
	{
		dis_list.push_back(enemy_dis);
	}

	if(dis_list_error_num >= dis_list_size)
	{
	dis_list_error_num = 0;
	dis_list.clear();
	}

	if(dis_list.size() >= dis_list_size)
	{
		//sum = accumulate(std::begin(dis_list),std::end(dis_list),0.0);
		for(int i = 0;i < dis_list.size(); i++)
		{
		sum += dis_list[i];
		}
		mean = sum/dis_list.size();
		if(fabs(enemy_dis - mean) > 0.250f)
		{
		enemy_dis = mean;
		dis_list_error_num++;
		}
		else
		{
		//enemy_dis = mean;
		double temp = dis_list[0];
		dis_list[0] = dis_list[1];
		dis_list[1] = dis_list[2];
		dis_list[2] = dis_list[3];
		dis_list[3] = dis_list[4];
		dis_list[4] = enemy_dis;
                //enemy_dis = (dis_list[0]+dis_list[1]+dis_list[2])/dis_list.size();
		}
	}

        armor.x = armio_center.x;
        armor.y = armio_center.y;
        armor.d = 1.17*enemy_dis*1000 ;

        armor.find_armio = true;
        armor.x = armor.d * (armor.x - cx) / fx ;
        armor.y = armor.d * (armor.y - cy) / fy;
        ROS_INFO (" x = [%f]",armor.x);
        ROS_INFO (" y = [%f]",armor.y);
        ROS_INFO (" d = [%f]",armor.d);

    }
    
    else
    {
	
        armor.x = 0.0;
        armor.y = 0.0;
        armor.d = 0.0;
        armor.find_armio = false;
    }
	
}



void ARMOR_DETECT::cal_armor_info(std::vector<cv::Point2f> &armor_points, cv::RotatedRect left_light, cv::RotatedRect right_light)
{
    cv::Point2f left_points[4], right_points[4];
    left_light.points(left_points);
    right_light.points(right_points);

    cv::Point2f right_lu, right_ld, left_ru, left_rd;
    for(int i = 0;i < 4;i++)
    {
	for( int j = i+1;j < 4;j++)
	{
	
	if(left_points[i].y > left_points[j].y)
	{
	Point2f tempLeft = left_points[i];
	
	left_points[i] = left_points[j];
	left_points[j] = tempLeft;
	}
	if(right_points[i].y > right_points[j].y)
	{
	Point2f tempRight = right_points[i];
	
	right_points[i] = right_points[j];
	right_points[j] = tempRight;
	}

        }
    }

    for(int i = 0;i < 4;i++)
    {
	cout<<"left_points["<<i<<"]"<<left_points[i]<<endl;
	cout<<"right_points["<<i<<"]"<<right_points[i]<<endl;
    }


    //std::sort(left_points, left_points + 4, [](const cv::Point2f & p1, const cv::Point2f & p2) { return p1.x < p2.x; });
   // std::sort(right_points, right_points + 4, [](const cv::Point2f & p1, const cv::Point2f & p2) { return p1.x < p2.x; });

/*
    if (right_points[0].y < right_points[3].y){
        right_lu = right_points[0];
        right_ld = right_points[3];
    }
    else{
        right_lu = right_points[3];
        right_ld = right_points[0];
    }

    if (left_points[0].y < left_points[3].y)	{
        left_ru = left_points[0];
        left_rd = left_points[3];
    }
    else {
        left_ru = left_points[3];
        left_rd = left_points[0];
    }
*/	
	//left_ru.x = left_ru.x - min(left_light.size.width,left_light.size.height)/2;
	//left_rd.x = left_rd.x + min(left_light.size.width,left_light.size.height)/2;
	//right_lu.x = right_lu.x - min(right_light.size.width,right_light.size.height)/2;
	//left_rd.x = left_rd.x + min(right_light.size.width,right_light.size.height)/2;
        right_lu = right_points[0];
        right_ld = right_points[3];
        left_ru = left_points[0];
        left_rd = left_points[3];

	left_ru.x = (left_ru.x + left_rd.x)/2;
        left_rd.x = left_ru.x;

	right_lu.x = (right_lu.x + right_ld.x)/2;
        right_ld.x = right_lu.x;
    	cout<<"left_ru = "<<left_ru<<endl;
	cout<<"right_lu = "<<right_lu<<endl;
	cout<<"right_ld = "<<right_ld<<endl;
	cout<<"left_rd = "<<left_rd<<endl;
    armor_points.push_back(left_ru );
    armor_points.push_back(right_lu);
    armor_points.push_back(right_ld);
    armor_points.push_back(left_rd );
}



