// roscpp
#include "ros/ros.h"

// Messages that I need
#include "sensor_msgs/LaserScan.h"
#include "geometry_msgs/PoseWithCovarianceStamped.h"
#include "geometry_msgs/PoseArray.h"
#include "geometry_msgs/Pose.h"
#include "nav_msgs/GetMap.h"
#include "nav_msgs/SetMap.h"
#include "std_srvs/Empty.h"

// For transform support
#include "tf/transform_broadcaster.h"
#include "tf/transform_listener.h"
#include "tf/message_filter.h"
#include "tf/tf.h"
#include "message_filters/subscriber.h"

#include "laser_detect/map.h"

#include <visualization_msgs/Marker.h>

typedef struct
{
   double x;
   double y;
   double hitLength;
   double yaw;
}LASER_DOT_Typedef;

typedef struct
{
   int Min_Yaw_Index;
   int Max_Yaw_Index;
   int count;
}REGION_Typedef;

typedef struct
{
   double MeanX;
   double MeanY;
   int count;
}UTILITY_REGION_Typedef;


class laser_detect
{
public:
    laser_detect();
    ~laser_detect();

    ros::Publisher* marker_pub;

    std::vector<UTILITY_REGION_Typedef> utility_obj_regions;

private:
    message_filters::Subscriber<sensor_msgs::LaserScan>* laser_scan_sub_;
    tf::MessageFilter<sensor_msgs::LaserScan>* laser_scan_filter_;


    bool laser_scan_solve(int count, tf::Stamped<tf::Pose> &laserPose, UTILITY_REGION_Typedef & PublishPoint);
    void laserReceived(const sensor_msgs::LaserScanConstPtr& laser_scan);
    void requesStatictMap();
    map_t* convertMap(const nav_msgs::OccupancyGrid& map_msg );
    bool iflaserDotsInRegion(const LASER_DOT_Typedef &curDots, const LASER_DOT_Typedef &preDots);

    map_t* static_map_= NULL;
    bool IfStaticMapRec = false;

    tf::TransformListener* tf_;
    tf::TransformBroadcaster* laser_detect_tf_;
    tf::Transform laser_latest_detect_tf_;

    //parameter for what base to use
    std::string base_frame_id_;
    std::string global_frame_id_;
    std::string laser_frame_id_;
    std::string detect_frame_id_;

    std::string scan_topic_;

    double (*laser_scan_range)[2];

    std::vector<LASER_DOT_Typedef> obj_detect_dots;
    std::vector<REGION_Typedef> obj_regions;

    UTILITY_REGION_Typedef PublishPoint;

    boost::recursive_mutex configuration_mutex_;

    ros::NodeHandle nh_;
    boost::mutex _mutex;
};
