#include <laser_detect/laser_detect.h>
#include<tf/transform_broadcaster.h>
#include <armor_detect/armor_msg.h>
#include <vector>
ros::Time current_time;
ros::Time last_time;

typedef struct
{
  float X;
  float Y;
  ros::Time record_time;
}CameraDetectTypedef;

std:: vector <CameraDetectTypedef> CameraDetectVect;
std:: vector <CameraDetectTypedef> RivalConfirmVect;
tf::StampedTransform CameraDetectStamped;

void ArmorInfoCallback(const armor_detect::armor_msg& armor_info)
{
    if(armor_info.detected == true)
    {
      tf::Transform CameraDetect = tf::Transform(tf::Quaternion(0, 0, 0, 1), tf::Vector3(armor_info.d,-armor_info.x, 0));
      CameraDetectStamped = tf::StampedTransform(CameraDetect,ros::Time::now(),"map","camera");
      CameraDetectTypedef CameraRivalObj;
      CameraRivalObj.X = CameraDetectStamped.getOrigin().getX();
      CameraRivalObj.Y = CameraDetectStamped.getOrigin().getX();
      CameraRivalObj.record_time = ros::Time::now();

      CameraDetectVect.push_back(CameraRivalObj);
    }
}




int main(int argc, char** argv)
{
  const double TimeOut = 1.0;
  const double CameraLaserMatchErr = 0.5;
  const int maxVectSize = 10;
  const double DisDetectTrack = 2;

  ros::init(argc, argv, "laser_detect");

  laser_detect laser_detect_obj;

  ros::NodeHandle nh;

  ros::Subscriber ArmorInfoSub;

  tf::TransformBroadcaster* Camera_detect_tf_ = new tf::TransformBroadcaster();
  tf::TransformBroadcaster* RivalConfirm_tf_ = new tf::TransformBroadcaster();

  double RivalTrackX = 0;
  double RivalTrackY = 0;
  bool IsTracking = false;
  uint8_t TrackLost = 0;

  ros::Rate loopRate(50);

  ArmorInfoSub = nh.subscribe("armor_info", 10, ArmorInfoCallback);
  while(ros::ok())
  {
     loopRate.sleep();

     ros::spinOnce();

     Camera_detect_tf_->sendTransform(CameraDetectStamped);

     int CameraRivalObjNum = CameraDetectVect.size();
     int TimeOutedIndex = -1;

     for(int i = 0;i < CameraRivalObjNum;i++)
     {
         if(ros::Time::now().toSec() - CameraDetectVect.at(i).record_time.toSec() > TimeOut)
         {
             TimeOutedIndex = i;
             continue;
         }
         else
         {
            if(!laser_detect_obj.utility_obj_regions.empty())
            {
                for(int j = 0;j < laser_detect_obj.utility_obj_regions.size();j++)
                {
                    double dx = CameraDetectVect.at(i).X - laser_detect_obj.utility_obj_regions.at(j).MeanX;
                    double dy = CameraDetectVect.at(i).Y - laser_detect_obj.utility_obj_regions.at(j).MeanY;
                    if(sqrt(dx*dx + dy*dy) < CameraLaserMatchErr)
                    {
                        CameraDetectTypedef RivalConfirm;
                        RivalConfirm.X = laser_detect_obj.utility_obj_regions.at(j).MeanX;
                        RivalConfirm.Y = laser_detect_obj.utility_obj_regions.at(j).MeanY;
                        RivalConfirm.record_time = CameraDetectVect.at(i).record_time;
                        RivalConfirmVect.push_back(RivalConfirm);
                        ROS_WARN("laser_camera confirmed rival x %f , y %f ",CameraDetectVect.at(i).X,CameraDetectVect.at(i).Y);
                    }
                }
            }
         }
     }

     if(RivalConfirmVect.size() == 0)
     {
         TrackLost ++ ;
         if(TrackLost > 254)
         {
             TrackLost = 0;
             IsTracking = false;
         }
     }
     else
     {
         TrackLost = 0;
     }

     if(TimeOutedIndex >= 0 && TimeOutedIndex < CameraDetectVect.size())
     {
         CameraDetectVect.erase(CameraDetectVect.begin(),CameraDetectVect.begin()+ TimeOutedIndex);
     }


//     for(int i = 0; i < TimeOutedIndex;i++)
//     {
//         CameraDetectVect.pop();
//     }

     double MinSqrt = FLT_MAX;

     if(IsTracking == false && RivalConfirmVect.size() != 0)
     {
         RivalTrackX = RivalConfirmVect.at(0).X;
         RivalTrackY = RivalConfirmVect.at(0).Y;
         IsTracking = true;
     }

     for(int i = 0;i < RivalConfirmVect.size();i++)
     {
         double ErrDis = sqrt((RivalConfirmVect.at(i).X - RivalTrackX) * (RivalConfirmVect.at(i).X - RivalTrackX) +
                              (RivalConfirmVect.at(i).Y - RivalTrackY) * (RivalConfirmVect.at(i).Y - RivalTrackY)) ;

         if(ErrDis < DisDetectTrack && ErrDis < MinSqrt)
         {
             RivalTrackX = RivalConfirmVect.at(i).X;
             RivalTrackY = RivalConfirmVect.at(i).Y;

             MinSqrt = ErrDis;
         }
     }

     int ConfirmOutdated = -1;
     for(int i = 0; i < RivalConfirmVect.size();i++)
     {
         if(ros::Time::now().toSec() - RivalConfirmVect.at(i).record_time.toSec() > TimeOut)
         {
             ConfirmOutdated = i;
         }
     }
     if(ConfirmOutdated >= 0 && ConfirmOutdated < RivalConfirmVect.size())
     {
        RivalConfirmVect.erase(RivalConfirmVect.begin(),RivalConfirmVect.begin() + ConfirmOutdated);
     }


     if(IsTracking)
     {
         RivalConfirm_tf_->sendTransform(tf::StampedTransform(tf::Transform(tf::Quaternion(0, 0, 0, 1), tf::Vector3(RivalTrackX,RivalTrackY, 0)),ros::Time::now(),"map","Confirmed"));
     }


     if(!IsTracking)
     {

     }

  }



  return(0);
}
