#include "logical_core.h"

LogicalCore::LogicalCore(tf::TransformListener &tf) : ac_("move_base", true), car_state_(IDLE_STATE), tf_(tf)
{
    fillPatrolList();
    p_mission_mode_ = p_patrol_mode_;
    ros::Rate loop(30);
    ac_.waitForServer();

    p_patrol_mode_  = new PatrolMode(this);
    p_mission_mode_ = p_patrol_mode_;

    while(ros::ok())
    {
        stateMachine();
        ros::spinOnce();
        loop.sleep();
    }
}

void LogicalCore::fillPatrolList()
{
    move_base_msgs::MoveBaseGoal temp_goal;
    temp_goal.target_pose.header.frame_id = "map";

    cv::FileStorage fs("stopblue1.yaml", cv::FileStorage::READ);
    if( !fs.isOpened() ) // if we have file with parameters, read them
    {
        std::cout<<"ERROR, cannot open stopblue1.yaml!"<<std::endl;
    }
    cv::FileNode list_n = fs["features"];
    cv::FileNodeIterator it = list_n.begin(), it_end = list_n.end();
    for (; it != it_end; ++it)
    {
        temp_goal.target_pose.pose.position.x   = (double)(*it)["x"];
        temp_goal.target_pose.pose.position.y   = (double)(*it)["y"];
        temp_goal.target_pose.pose.orientation.z = (double)(*it)["z"];
        temp_goal.target_pose.pose.orientation.w = (double)(*it)["w"];
        std::cout << (double)(*it)["x"] << "  ";
        std::cout << (double)(*it)["y"] << std::endl;
        //temp_goal.target_pose.pose.orientation.w = 1.;
        patrol_list_.push_back(temp_goal);
    }
    std::reverse(patrol_list_.begin(), patrol_list_.end());
    fs.release();
}

double LogicalCore::distance(const geometry_msgs::PoseStamped& p1, const geometry_msgs::PoseStamped& p2)
{
    return hypot(p1.pose.position.x - p2.pose.position.x, p1.pose.position.y - p2.pose.position.y);
}

void LogicalCore::stateMachine()
{
    switch(car_state_)
    {
    case OBTAINED_GOAL_STATE:
    {
        p_mission_mode_->obtainedGoalState();
        break;
    }
    case BUSY_STATE:
    {
        p_mission_mode_->busyState();
        break;
    }
    case IDLE_STATE:
    {
        p_mission_mode_->idleState();
        break;
    }
    }
}
