 #include <ros/ros.h>
 #include <nav_msgs/Path.h>
 #include <costmap_2d/costmap_2d_ros.h>
 #include <costmap_2d/costmap_2d.h>
 #include <nav_core/base_global_planner.h>
 #include <geometry_msgs/PoseStamped.h>
#include <my_global_planner/orientation_filter.h>
 #include <my_global_planner/DijkPlanner.h>

 using std::string;

 #ifndef GLOBAL_PLANNER_CPP
 #define GLOBAL_PLANNER_CPP

 namespace my_global_planner
 {

 class MyGlobalPlanner : public nav_core::BaseGlobalPlanner
 {
 public:

  MyGlobalPlanner();
  MyGlobalPlanner(std::string name, costmap_2d::Costmap2DROS* costmap_ros);
  ~MyGlobalPlanner();

  /** overridden classes from interface nav_core::BaseGlobalPlanner **/
  void initialize(std::string name, costmap_2d::Costmap2DROS* costmap_ros);
  bool makePlan(const geometry_msgs::PoseStamped& start,
                const geometry_msgs::PoseStamped& goal,
                std::vector<geometry_msgs::PoseStamped>& plan
               );

   DijkPlanner* myPlanner;

 private:
   void publishPotential(float const* SeekRegion);
   void publishPlan(const std::vector<geometry_msgs::PoseStamped>& globalplan);
   bool PathToGlobalPlan(std::vector<std::pair<float, float> >& path,std::vector<geometry_msgs::PoseStamped>& globalplan);

   void mapToWorld(double map_x, double map_y, double& world_x, double& world_y);
   bool worldToMap(double world_x, double world_y, double& map_x, double& map_y);

   OrientationFilter* pOrientationCal;

   std::vector<std::pair<float, float> > path;

   ros::Publisher plan_pub_;
   ros::Publisher potential_pub_;

   boost::mutex _mutex;

   bool IsInitialized;
   std::string frame_id_;
   std::string tf_prefix_;
   costmap_2d::Costmap2D* costmap_;
   unsigned char* CostmapIndexArray;

   //f(n) = g(n) + heuristic_weight * h(n)
   float heuristic_weight_;

  };
 };
 #endif
