#include <ros/ros.h>
#include <costmap_2d/costmap_2d.h>
#include <my_global_planner/DijkPlanner.h>
#include <costmap_2d/cost_values.h>
#include <cmath>
#include <queue>
#include <my_global_planner/gradient_path.h>


using namespace std;
#define LETHAL_COST   100

//Default Constructor
namespace my_global_planner {

DijkPlanner::DijkPlanner(costmap_2d::Costmap2D* costmap)
{
  ros::NodeHandle private_nh("~/");
  private_nh.param("use_gradient_construct",use_gradient_construct_,true);

  Size_X = costmap->getSizeInCellsX();
  Size_Y = costmap->getSizeInCellsY();

  NodesCount = Size_X * Size_Y;

  Node_cost_dist = new float[NodesCount];
  PrevNode = new unsigned int[NodesCount];
  NodeIsVisted    = new bool[NodesCount];

  CostmapIndexArray = costmap->getCharMap();

  lethal_cost_ = LETHAL_COST;

  //ROS_WARN("use gradient");
  gradientConstruct = new GradientPath(Size_X,Size_Y,lethal_cost_);

}

DijkPlanner::~DijkPlanner()
{
  delete Node_cost_dist;
  delete PrevNode;
  delete NodeIsVisted;

  delete gradientConstruct;
}

void DijkPlanner::DrawMapOutLine(unsigned char* costarr, int nx, int ny, unsigned char value)
{
      unsigned char* pc = costarr;
      for (int i = 0; i < nx; i++)
          *pc++ = value;
      pc = costarr + (ny - 1) * nx;
      for (int i = 0; i < nx; i++)
          *pc++ = value;
      pc = costarr;
      for (int i = 0; i < ny; i++, pc += nx)
          *pc = value;
      pc = costarr + nx - 1;
      for (int i = 0; i < ny; i++, pc += nx)
          *pc = value;
  }



//#define A_STAR
#define QUADRATIC

#define DIST_FACTOR 10

 #ifdef A_STAR
 #define A_STAR_DIST_TO_GOAL_FACTOR  10
 #endif

  bool DijkPlanner::Alogrithim_OnCostMap(unsigned int start_X,unsigned int start_Y,unsigned int goal_X,unsigned int goal_Y)
  {
    typedef pair<float,int> DIST_INDEX_Typedef;
    priority_queue<DIST_INDEX_Typedef,vector<DIST_INDEX_Typedef>,greater<DIST_INDEX_Typedef> >NextNodeQueue;

    unsigned int StartIndex = XY_TO_INDEX(start_X,start_Y);
    unsigned int GoalIndex  = XY_TO_INDEX(goal_X ,goal_Y);

    for(int i = 0; i < NodesCount; ++i)
    {
        Node_cost_dist[i] = FLT_MAX;
        PrevNode[i] = PREV_UNDEFINED;
    }

    Node_cost_dist[StartIndex] = 0;
    #ifndef A_STAR
    NextNodeQueue.push(DIST_INDEX_Typedef(0,StartIndex));
    #else
    NextNodeQueue.push(DIST_INDEX_Typedef(A_STAR_DIST_TO_GOAL_FACTOR *(abs(goal_X - start_X)+abs(goal_Y - start_Y)),StartIndex));
    #endif


    // djikstra search
    while(!NextNodeQueue.empty())
    {
      //extract min distance element in next queue
      DIST_INDEX_Typedef CurrentNode = NextNodeQueue.top();
      NextNodeQueue.pop();

      unsigned int CurrentIndex = CurrentNode.second;
      float cur_cost_dist = CurrentNode.first;
      //maybe there are many dists of one nodes
      if(Node_cost_dist[CurrentIndex] < cur_cost_dist)
      {
        continue;
      }
      //end extract min distance element in next queue

      //reach the goal
      if(GoalIndex == CurrentIndex)
      {
        return true;
      }

      //neighbor
      for(int i = -1; i <= 1; i++)
      {
        for(int j = -1; j<=1;j++)
        {
          if(i == 0 && j == 0)
          {
            continue;
          }
          if(i*j != 0)
          {
            continue;
          }

          unsigned int NextIndex = CurrentIndex + i + j * Size_X;
          if(NextIndex >= NodesCount)
          {
            continue;
          }
          //calculate distance and cost
          unsigned char next_cost_inc = CostmapIndexArray[NextIndex];

          if(next_cost_inc >= lethal_cost_)
          {
            continue;
          }

          float next_dist_inc;
          if(i != 0 && j != 0)
          {
            next_dist_inc = SQRT2;
          }
          else
          {
            next_dist_inc = 1;
          }


          #ifdef A_STAR
          unsigned int NextNodeX = NextIndex% Size_X;
          unsigned int NextNodeY = NextIndex/ Size_X;
          float dist_to_goal = abs(NextNodeX - goal_X) + abs(NextNodeY - goal_Y);
          float next_cost_dist = next_dist_inc + next_cost_inc + cur_cost_dist + dist_to_goal * A_STAR_DIST_TO_GOAL_FACTOR ;//length cur_next = next_dist_inc + next_cost_inc
          #elif defined(QUADRATIC)
          unsigned int nearestIndex;
          float next_cost_dist_inc = next_dist_inc * DIST_FACTOR + next_cost_inc;
          float next_cost_dist = QuadraticCalculate_cost_dist(Node_cost_dist,next_cost_dist_inc,NextIndex,nearestIndex);
          #else

          float next_cost_dist = next_dist_inc  * DIST_FACTOR + next_cost_inc + cur_cost_dist;//length cur_next = next_dist_inc + next_cost_inc

          #endif
          //end calculate distance and cost

          if(next_cost_dist < Node_cost_dist[NextIndex])
          {
            Node_cost_dist[NextIndex] = next_cost_dist;
#ifdef QUADRATIC
//            if(CurrentIndex != nearestIndex)
//            {
//                ROS_WARN("CurIndex %d Neaest %d",CurrentIndex,nearestIndex);
//            }
            PrevNode[NextIndex] = nearestIndex;
#else
            PrevNode[NextIndex] = CurrentIndex;
#endif
            NextNodeQueue.push(DIST_INDEX_Typedef(next_cost_dist,NextIndex));
          }
        }
      }
    }
    return false;
  }



// bool DijkPlanner::Alogrithim_OnCostMap(unsigned int start_X,unsigned int start_Y,unsigned int goal_X,unsigned int goal_Y)
// {
//   ROS_WARN("int start_X  %d  start_Y  %d",start_X,start_Y);
//   ROS_WARN("int goal_X  %d  goal_Y  %d",goal_X,goal_Y);
//   unsigned int VisitedCount = 0;
//
//   unsigned int StartIndex = XY_TO_INDEX(start_X,start_Y);
//   unsigned int GoalIndex  = XY_TO_INDEX(goal_X ,goal_Y);
//
//   ROS_WARN("int StartIndex  %d  GoalIndex  %d",StartIndex,GoalIndex);
//
//   for(int i = 0; i < NodesCount; ++i)
//   {
//       Node_cost_dist[i] = FLT_MAX;
//       PrevNode[i] = PREV_UNDEFINED;
//       NodeIsVisted[i] = false;
//   }
//
//   Node_cost_dist[StartIndex] = 0;
//
//   unsigned int CurrentIndex;
//   // djikstra search
//   while(VisitedCount < NodesCount)
//   {
//     ROS_WARN("start search");
//     //search min distance element if unvisited
//     float min_cost_dist = FLT_MAX;
//     for(int i = 0; i < NodesCount; i++)
//     {
//         if(Node_cost_dist[i] < min_cost_dist && NodeIsVisted[i] == false)
//         {
//           min_cost_dist = Node_cost_dist[i];
//           CurrentIndex = i;
//         }
//     }
//     ROS_WARN("Minim dist searched  %d ",CurrentIndex);
//     NodeIsVisted[CurrentIndex] = true;
//     VisitedCount++;
//     float cur_cost_dist = min_cost_dist;//cur_cost_dist = Node_cost_dist[CurrentX][CurrentY]
//     //end search min distance element if unvisited
//
//     //reach the goal
//     if(GoalIndex == CurrentIndex)
//     {
//       return true;
//     }
//
//     for(int i = -1; i <= 1; i++)
//     {
//       for(int j = -1; j<=1;j++)
//       {
//         if(i == 0 && j == 0)
//         {
//           continue;
//         }
//
//         unsigned int NextIndex = CurrentIndex + i + j * Size_X;
//         if(NextIndex >= NodesCount)
//         {
//           continue;
//         }
//         //calculate distance and cost
//         unsigned char next_cost_inc = CostmapIndexArray[NextIndex];
//
//         if(next_cost_inc >= lethal_cost_)
//         {
//           continue;
//         }
//
//         float next_dist_inc;
//         if(i != 0 && j != 0)
//         {
//           next_dist_inc = SQRT2;
//         }
//         else
//         {
//           next_dist_inc = 1;
//         }
//
//         float next_cost_dist = next_dist_inc + next_cost_inc + cur_cost_dist;//length cur_next = next_dist_inc + next_cost_inc
//
//         //end calculate distance and cost
//         if(next_cost_dist < Node_cost_dist[NextIndex])
//         {
//           Node_cost_dist[NextIndex] = next_cost_dist;
//           PrevNode[NextIndex] = CurrentIndex;
//         }
//       }
//     }
//   }
//   return false;
// }


bool DijkPlanner::PathConstruct(unsigned int start_X,unsigned int start_Y,unsigned int goal_X,unsigned int goal_Y,std::vector<std::pair<float, float> >& path)
{
  path.clear();
  std::pair<unsigned int, unsigned int>path_end(goal_X,goal_Y);
  path.push_back(path_end);

  unsigned int StartIndex = XY_TO_INDEX(start_X,start_Y);
  unsigned int GoalIndex  = XY_TO_INDEX(goal_X ,goal_Y);

  unsigned int CurrentIndex = GoalIndex;

  for(unsigned i = 0;i < NodesCount; i++)
  {
    if(CurrentIndex  == StartIndex)
    {
        return true;
    }
    unsigned int prevIndex = PrevNode[CurrentIndex];

    if(prevIndex == PREV_UNDEFINED)
    {
      break;
    }
    std::pair<unsigned int, unsigned int>Prev(prevIndex%Size_X,prevIndex/Size_X);
    path.push_back(Prev);
    CurrentIndex = prevIndex;
  }
  return false;
}


bool DijkPlanner::global_path_seek(float start_X,float start_Y,float goal_X,float goal_Y,std::vector<std::pair<float, float> >& path)
{

   DrawMapOutLine(CostmapIndexArray, Size_X, Size_Y, costmap_2d::LETHAL_OBSTACLE);
  //ROS_WARN("Drawed Map bound ");
   if(Alogrithim_OnCostMap((unsigned int)round(start_X),(unsigned int)round(start_Y),(unsigned int)round(goal_X),(unsigned int)round(goal_Y)) == false)
   {
     return false;
   }
   if(use_gradient_construct_)
   {
      //ROS_WARN("use gradient");
      if(!gradientConstruct ->getPath(Node_cost_dist,(unsigned int)round(start_X),(unsigned int)round(start_Y),(unsigned int)round(goal_X),(unsigned int)round(goal_Y),path))
      {
          return false;
      }
   }
   else
   {
       //ROS_WARN("use search tree");
       if(PathConstruct((unsigned int)round(start_X),(unsigned int)round(start_Y),(unsigned int)round(goal_X),(unsigned int)round(goal_Y),path) == false)
       {
         return false;
       }
   }
   //ROS_WARN("path constructed");


   return true;
}


float DijkPlanner::QuadraticCalculate_cost_dist(const float * Node_cost_dist,unsigned char cell_cost,unsigned int NextIndex,unsigned int& index)
 {
     unsigned int ColMinIndex = NextIndex-1,RowMinIndex = NextIndex - Size_X,MinIndex;
     float ColMin = Node_cost_dist[ColMinIndex],RowMin = Node_cost_dist[RowMinIndex],Min_Col_Row_Diff;

     if(ColMin > Node_cost_dist[NextIndex + 1])
     {
         ColMin = Node_cost_dist[NextIndex + 1];
         ColMinIndex = NextIndex + 1;
     }
     if(RowMin > Node_cost_dist[NextIndex + Size_X])
     {
        RowMin = Node_cost_dist[NextIndex + Size_X];
        RowMinIndex = NextIndex + Size_X;
     }

     if(RowMin > ColMin)
     {
         MinIndex = ColMinIndex;
         Min_Col_Row_Diff = RowMin - ColMin;
     }
     else
     {
         MinIndex = RowMinIndex;
         Min_Col_Row_Diff = ColMin - RowMin;
     }

     index = MinIndex;

     if(Min_Col_Row_Diff > cell_cost)
     {
         return cell_cost + Node_cost_dist[index];
     }
     else
     {
         float rate = Min_Col_Row_Diff / cell_cost;
         float coff = -0.2301 * rate * rate + 0.5307 * rate + 0.7040;
         return Node_cost_dist[index] + coff * cell_cost;
     }
 }


};
