#include <ros/ros.h>
#include <move_base_msgs/MoveBaseAction.h>
#include <actionlib/client/simple_action_client.h>
#include <tf/transform_broadcaster.h>
#include <sstream>

#include <dynamic_reconfigure/server.h>
#include <send_goal/sendGoals_Config.h>

typedef actionlib::SimpleActionClient<move_base_msgs::MoveBaseAction> MoveBaseClient;

void callback(send_goal::sendGoals_Config &config, uint32_t level) {
	MoveBaseClient ac("move_base", true);
	move_base_msgs::MoveBaseGoal goal;

	ROS_INFO("Reconfigure Request: %f %f %f", 
		config.position_x, config.position_y, config.orientation_w); 

	while(!ac.waitForServer(ros::Duration(5.0))){
		ROS_INFO("Waiting for the move_base action server");
	}

	goal.target_pose.header.frame_id = "map";
	goal.target_pose.header.stamp = ros::Time::now();

	printf("config.position_x:  %f\n",config.position_x);
	printf("config.position_y:  %f\n",config.position_y);
	printf("config.orientation_w:  %f\n",config.orientation_w);

	goal.target_pose.pose.position.x = config.position_x;
	goal.target_pose.pose.position.y = config.position_y;
	goal.target_pose.pose.orientation.w = config.orientation_w;

	ROS_INFO("Sending goal1");
	ac.sendGoal(goal);

	ac.waitForResult();

	if(ac.getState() == actionlib::SimpleClientGoalState::SUCCEEDED)
		ROS_INFO("You have arrived to the goal position");
	else{
		ROS_INFO("The base failed for some reason");
	}
}


int main(int argc, char** argv){
	ros::init(argc, argv, "navigation_goals");

	dynamic_reconfigure::Server<send_goal::sendGoals_Config> server;
	dynamic_reconfigure::Server<send_goal::sendGoals_Config>::CallbackType f;

	f = boost::bind(callback, _1, _2);
	server.setCallback(f);

	ROS_INFO("Spinning node");
	ros::spin();
	return 0;
}
