#include "comm_read.h"
#include <ros/ros.h>

namespace autocar
{
namespace serial_mul
{

comm_read::comm_read(const std::string& devel)
{
  //std::string serial_port;
  data_len = sizeof(data_t);
  dev = devel;
  serial_baudrate = B115200;
}

bool comm_read::read_setup()
{
  fd = open (dev.c_str(), O_RDWR);
  if(fd==-1)
  {
    //perror("error!");
    return false;
  }
  struct termios oldtio;
  tcgetattr(fd,&oldtio);
  if(tcgetattr(fd,&oldtio)!=0)
  {
    //perror("error!");
    return false;
  }
  struct termios newtio;
  newtio.c_cflag=CIBAUD|CRTSCTS|CS8|CLOCAL|CREAD;
  cfsetispeed(&newtio,serial_baudrate);
  cfsetospeed(&newtio,serial_baudrate);
  newtio.c_iflag=IGNPAR;
  newtio.c_oflag=0;
  newtio.c_lflag=0;
  newtio.c_cc[VMIN]=1;
  newtio.c_cc[VTIME]=0;
  tcflush(fd,TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
  return true;
}

void comm_read::read_init()
{

}

#define HEAD_BYTE 0xA5
#define TAIL_BYTE 0xFE

void comm_read::FrameConstruct()
{
  ROS_WARN("start    ");
  int readlength = read(fd,&buff[BufferIndex],255);

  if(readlength <= 0)
  {
      ROS_WARN("length  %d",readlength);
      return;
  }

  BufferIndex += readlength;

  ROS_WARN("BufferIndex %d",BufferIndex);

  for(;HeadIndex < BufferIndex;HeadIndex++)
  {
      if(buff[HeadIndex] == HEAD_BYTE)//head detected
      {
          ROS_WARN("head detected %d",HeadIndex);

          uint16_t TailIndex = HeadIndex + sizeof(data_t) - 1;
          if(TailIndex < BufferIndex)//can do length check
          {
              ROS_WARN("tail check");
              if(buff[TailIndex] == TAIL_BYTE)   //integrated frame
              {
                  memcpy(&data,&buff[HeadIndex],sizeof(data_t));
                  Frame_read();
                  HeadIndex = TailIndex;
                  ROS_WARN("data received");
              }//maybe there is not only one frame
              //don't comply tail - end = length,then next byte check
          }
          else
          {
              ROS_WARN("a head detected,but can't do length check");
              break;
          }
      }
  }

  if(HeadIndex > 255)
  {
      memcpy(&buff[0],&buff[HeadIndex],(BufferIndex - HeadIndex)*sizeof(buff[0]));
      BufferIndex -= HeadIndex;
      HeadIndex = 0;
  }
}

void comm_read::read_data()
{
    FrameConstruct();
}

void comm_read::Frame_read()
{
//    if(!If_yaw_inited)
//    {
//      init_yaw = data.angle;

//      tcflush(fd,TCIFLUSH);

//      If_yaw_inited = true;
//    }
//    else
    {
        Y             = ( data.angle - init_yaw );
        YawAngle      = Y*(M_PI/180.);

        ForeBack      = data.v_x;
        LeftRight     = data.v_y;

        R             = data.v_w;
        Rotate        = R*(M_PI/180.);
        pubData.angle = YawAngle;
        pubData.v_x   = ForeBack/1000.;
        pubData.v_y   = LeftRight/1000.;
        pubData.v_r   = Rotate;
        pubData.yaw   = -data.pan_title_yaw*(M_PI/180);
        shoot_data.bloodinfo = data.blood;
        shoot_data.datastatus = data.data_status;
        pubData.x     = data.x/1000;
        pubData.y     = data.y/1000;


        CarDataUpdated = true;

    }
}

comm_read::~comm_read()
{
  close(fd);
}

}//namespace serial_comm
}//namespace autocar
