#include<ros/ros.h>
#include<tf/transform_broadcaster.h>
#include "tf_factory.h"
#include <serial_comm/car_speed.h>

autocar::tf_mul::tf_factory camera_tf;

void ShootYawCallback(const serial_comm::car_speed shootYaw)
{
  ROS_WARN("YAW %f",shootYaw.yaw);
  camera_tf.SetYawQuaternion(shootYaw.yaw);
}


int main(int argc, char** argv)
{
    ros::init(argc, argv, "robot_tf_publisher");
    ros::NodeHandle n;
    ros::Rate r(10);
    tf::TransformBroadcaster broadcaster;
    autocar::tf_mul::tf_factory lidar_tf;
    lidar_tf.create_tf(-0.1, 0.0, 0.1);
    tf::Quaternion lidar_quaternion = lidar_tf.get_rotate();
    tf::Vector3 lidar_trans = lidar_tf.get_transf();
    
    ROS_WARN("subscribe");
    ros::Subscriber ShootYawSub = n.subscribe("car_info", 10, ShootYawCallback);

    autocar::tf_mul::tf_factory shoot_base_tf;
    shoot_base_tf.create_tf(0.06, 0.0, 0);
    tf::Quaternion shoot_base_quaternion = shoot_base_tf.get_rotate();
    tf::Vector3 shoot_base_trans = shoot_base_tf.get_transf();


    camera_tf.create_tf(0.176, 0.0,0);
    tf::Vector3 camera_trans = camera_tf.get_transf();

    while(n.ok())
    {
        broadcaster.sendTransform(tf::StampedTransform(tf::Transform(lidar_quaternion, lidar_trans), ros::Time::now(), "base_link", "laser"));
        broadcaster.sendTransform(tf::StampedTransform(tf::Transform(shoot_base_quaternion, shoot_base_trans), ros::Time::now(), "base_link", "shoot_base"));

        tf::Quaternion camera_quaternion = camera_tf.get_rotate();

        broadcaster.sendTransform(tf::StampedTransform(tf::Transform(camera_quaternion, camera_trans), ros::Time::now(), "shoot_base", "camera"));
        r.sleep();
    }
}
