#ifndef FRAME_H
#define FRAME_H

#include <iostream>
#include <cstring>
#include <vector>
#include <list>
#include <sys/time.h>
#include <ros/ros.h>
#include <cv_bridge/cv_bridge.h>
#include <std_msgs/Char.h>

#ifndef __APPLE__
#define EXPOSURE_CONTROL // only works in Linux
#endif

#ifdef EXPOSURE_CONTROL
#include <libv4l2.h>
#include <linux/videodev2.h>
#include <fcntl.h>
#include <errno.h>
#endif
#include "opencv2/opencv.hpp"

// April tags detector and various families that can be selected by command line option
#include "AprilTags/TagDetector.h"
#include "AprilTags/Tag16h5.h"
#include "AprilTags/Tag25h7.h"
#include "AprilTags/Tag25h9.h"
#include "AprilTags/Tag36h9.h"
#include "AprilTags/Tag36h11.h"
#include "opencv2/core/core.hpp"
#include "geometry_msgs/Twist.h"
#include "tf/transform_broadcaster.h"
#include <unistd.h>
#include "Serial.h"
#include "ros/ros.h"

#include <cmath>

#ifndef PI
const double PI = 3.14159265358979323846;
#endif
const double TWOPI = 2.0*PI;

extern int optind;
extern char *optarg;

using namespace std;
using namespace cv;
class Demo
{
public:
	Demo();
	void setTagCodes(string s);
	void setup(ros::NodeHandle *nh, bool b_draw, bool b_timing, string Tag_Name);
// 	void setup(ros::NodeHandle *nh, bool b_draw, bool b_timing, string Tag_Name);
	void print_detection(AprilTags::TagDetection& detection, char signal);
	void JGGLocalization(cv::Mat & image, cv::Mat image_gray, char DollColor);
	void Localization(cv::Mat & image);
	void ColorLocalization(cv::Mat image, char DollColor);
	void SquareLocalization(cv::Mat image, cv::Mat image_gray);
	void processImage(cv::Mat& image_gray, char signal);
	double tic();
	void RectifyFourPoints(vector<cv::Point2f>& ori);
	
	void wRo_to_euler(const Eigen::Matrix3d& wRo, double& yaw, double& pitch, double& roll);
	inline double standardRad(double t); 
	float distance(Point2f a, Point2f b);
	void getFourInsideCorners(vector<pair<cv::Rect, vector<cv::Point2f> > > lineRects,vector<cv::Point2f >& squareCorners);
	
	void ColorSegment(Mat &image,Mat &imgThresholded,char DollColor);
	void getPosition(double tag_size, char cameraId);
	
// 	static void floodFillPostprocess(Mat& img, const Scalar& colorDiff = Scalar::all(1)); 
	Point cannyArea(Mat &imgThresholded);
	
	double mediaBlur(vector<double > filter_value);
	void filterClear();
	
	int detect_led(Mat& img, char* ret);
	void Led_Recognition(Mat &img);
	
	static bool ValueComp( double a, double b){
		return a < b;
	}
	
	AprilTags::TagDetector* m_tagDetector;
	AprilTags::TagDetector* m_tagDetector_16h5;
	AprilTags::TagCodes m_tagCodes;
	
	ros::Publisher tag_publisher;
// 	tf::TransformBroadcaster br;
	
	bool m_draw; // draw image and April tag detections?
	bool m_timing; // print timing information for each tag extraction call

	bool m_8_checked, m_4_checked, m_7_checked;
	
	int m_width; // image size in pixels
	int m_height;
	double m_tagSize; // April tag side length in meters of square black frame
	double m_fx; // camera focal length in pixels
	double m_fy;
	double m_px; // camera principal point
	double m_py;
	
	double m_color_fx;
	double m_color_fy;
	double m_color_px; // camera principal point
	double m_color_py;
	
	double m_roll, m_yaw, m_pitch;
	Eigen::Vector3d m_translation;
	int m_id;
	
// 	vector<Eigen::Vector3d> filter_translation;
	vector<double > filter_x;
	vector<double > filter_y;
	vector<double > filter_z;
	vector< double> filter_yaw;

	int m_deviceId; // camera id (in case of multiple cameras)

	list<string> m_imgNames;

	cv::VideoCapture m_cap;

	int m_exposure;
	int m_gain;
	int m_brightness;
	
	bool DEBUG;

	Serial m_serial;
	
	vector<Point2f> centerRect;
	
	
	
// 	ros::NodeHandle m_NH;
	
};



#endif
