#include <ros/ros.h>
#include "std_msgs/String.h"
#include <geometry_msgs/Vector3Stamped.h>
#include <geometry_msgs/PoseStamped.h>
#include <geometry_msgs/PointStamped.h>
#include <geometry_msgs/TwistStamped.h>
#include "uav_mission/uav_mission_msg.h"
#include <tf/tf.h>
#include "math.h"
#include <fstream>
#include <iostream>
#include <dji_sdk/dji_drone.h>
#include <std_msgs/UInt8.h>
using namespace std;


//enum uav_state{
// standing_by, busy
//}current_state;
std_msgs::UInt8 current_state;

geometry_msgs::PoseStamped tag_pose;
geometry_msgs::PoseStamped apron_pose;
geometry_msgs::PoseStamped guidance_pose;
//feedback

geometry_msgs::PoseStamped goal_pose,zero_pose,origin_pose;
uav_mission::uav_mission_msg current_mission,last_mission;
//goal & const

geometry_msgs::TwistStamped velocity_ctrl;
//control

float x_error0,x_error1;
float y_error0,y_error1;
float z_error0,z_error1;
float yaw_error0,yaw_error1,yaw_error2;
float x_error_int=0,y_error_int(0),z_error_int(0),yaw_error_int(0);
//error

double delt_t;
//t

float Kp_pos,Ki_pos,Kd_pos;
float Kp_height,Ki_height,Kd_height;
float Kp_yaw,Ki_yaw,Kd_yaw;
//gain

float XY_ERROR_INT_LIMIT=2;
float XY_V_CTRL_LIMIT=4;
float Z_ERROR_INT_LIMIT=0.25;
float Z_V_CTRL_LIMIT=0.5;
float YAW_ERROR_INT_LIMIT=100;
float YAW_V_CTRL_LIMIT=90;
//limit

float goal_x[3],goal_y[3],goal_z[3],goal_yaw[3];
float apron_hover_height;
//for pid tuning

ros::Time lastpose_stamp(0);
ros::Time take_off_stamp(0);
const ros::Time zero_time_stamp(0);
ros::Time guidancepose_stamp(0),tagpose_stamp(0),apronpose_stamp(0);
//time stamp

bool flight_log(false);
string log_dir="~/flight_log/log.txt";
ofstream outfile;
//log

void guidance_pose_cb(const geometry_msgs::PoseStamped& guidance_pose_)
{
  guidance_pose.header.frame_id = guidance_pose_.header.frame_id;
  guidance_pose.header.stamp    = guidance_pose_.header.stamp;
  guidance_pose.pose          = guidance_pose_.pose;

  guidancepose_stamp            = guidance_pose_.header.stamp;

 // ROS_INFO("NEW pos");
}

void tag_pose_cb(const geometry_msgs::PoseStamped & tag_pose_)
{
  tag_pose.header.frame_id =tag_pose_.header.frame_id;
  tag_pose.header.stamp    =tag_pose_.header.stamp;
  //摄像头:下Y 右X 前Z
  tag_pose.pose.position.x      =tag_pose_.pose.position.y;
  tag_pose.pose.position.y      =-tag_pose_.pose.position.x;
  tag_pose.pose.position.z      =-tag_pose_.pose.position.z;
  tag_pose.pose.orientation     =tag_pose_.pose.orientation;
  //作一个坐标变换.
  tagpose_stamp=tag_pose_.header.stamp;

 // outfile << current_pos.header.stamp.toSec()-firstpos_stamp.toSec()<<" "<<current_pos.vector.x<<" "<<current_pos.vector.y<<" "<<current_pos.vector.z<<std::endl;
 // ROS_INFO("NEW pos");
}


void apron_pose_cb(const geometry_msgs::PoseStamped & apron_pose_)
{
  apron_pose.header.frame_id =apron_pose_.header.frame_id;
  apron_pose.header.stamp    =apron_pose_.header.stamp;
  //摄像头:下Y 右X 前Z
  apron_pose.pose.position.x          =apron_pose_.pose.position.y;
  apron_pose.pose.position.y          =-apron_pose_.pose.position.x;
  apron_pose.pose.position.z          =-apron_pose_.pose.position.z;
  apron_pose.pose.orientation         =tf::createQuaternionMsgFromYaw(0);
  //作一个坐标变换.
  apronpose_stamp=apron_pose_.header.stamp;

  // outfile << current_pos.header.stamp.toSec()-firstpos_stamp.toSec()<<" "<<current_pos.vector.x<<" "<<current_pos.vector.y<<" "<<current_pos.vector.z<<std::endl;
  // ROS_INFO("NEW pos");
}

void goal_pose_cb(const geometry_msgs::PoseStamped& goal_pose_)
{//改成goal+oringin
  goal_pose.header.frame_id	=goal_pose_.header.frame_id;
  goal_pose.header.stamp		=goal_pose_.header.stamp;
  goal_pose.pose            =goal_pose_.pose;
}
void uav_mission_msg_cb(const uav_mission::uav_mission_msg& mission_msg_)
{
    ROS_INFO("new mission:%d",mission_msg_.MISSION);
    last_mission.MISSION=current_mission.MISSION;
    current_mission.MISSION=mission_msg_.MISSION;
}
void pidcontrol_reinit()
{
   x_error_int=0;
   y_error_int=0;
   z_error_int=0;
   yaw_error_int=0;
   x_error0=0;
   x_error1=0;
   y_error0=0;
   y_error1=0;
   z_error0=0;
   z_error1=0;
   yaw_error0=0;
   yaw_error1=0;
   lastpose_stamp=lastpose_stamp-(lastpose_stamp-zero_time_stamp);
}


/*   pidcontrol  @ m/s && deg/s  */
void pidcontrol_pose(const  geometry_msgs::PoseStamped& goal_pose_,\
                     const  geometry_msgs::PoseStamped& current_pose_,\
                     geometry_msgs::TwistStamped& velocity_ctrl_)
{
  if (!((Kp_pos<=0. && Ki_pos<=0. && Kd_pos<=0.) || (Kp_pos>=0. && Ki_pos>=0. && Kd_pos>=0.)) ) // All 3 gains should have the same sign
  {
      ROS_WARN("All three gains (Kp_pos, Ki_pos, Kd_pos) should have the same sign for stability.");
      return;
  }//可以修改

  if(!lastpose_stamp.isZero())
  {

    delt_t=current_pose_.header.stamp.toSec()-lastpose_stamp.toSec();
    lastpose_stamp=current_pose_.header.stamp;
  }
  else
  {//first time in
    lastpose_stamp=current_pose_.header.stamp;
    x_error0=goal_pose_.pose.position.x-current_pose_.pose.position.x;
    y_error0=goal_pose_.pose.position.y-current_pose_.pose.position.y;
    z_error0=goal_pose_.pose.position.z-current_pose_.pose.position.z;

    ROS_INFO("FIRST IN PID");

    ROS_INFO("FIRST IN PID");

    ROS_INFO("FIRST IN PID");
    return;
  }

  /*  x  */

  x_error1=x_error0;
  x_error0=goal_pose_.pose.position.x-current_pose_.pose.position.x;
  //error integral
  x_error_int+=x_error0*delt_t;
  // Apply windup limit to limit the size of the integral term
  if ( x_error_int > XY_ERROR_INT_LIMIT)
    x_error_int = XY_ERROR_INT_LIMIT;
  if ( x_error_int < -XY_ERROR_INT_LIMIT)
    x_error_int = -XY_ERROR_INT_LIMIT;

  velocity_ctrl_.twist.linear.x=Kp_pos*x_error0+Kd_pos*(x_error0-x_error1)/delt_t+Ki_pos*x_error_int;
  //limit  x  velocity_ctrl_
  if(velocity_ctrl_.twist.linear.x>XY_V_CTRL_LIMIT)
     velocity_ctrl_.twist.linear.x=XY_V_CTRL_LIMIT;
  if(velocity_ctrl_.twist.linear.x<-XY_V_CTRL_LIMIT)
     velocity_ctrl_.twist.linear.x=-XY_V_CTRL_LIMIT;

  /*  y  */

  y_error1=y_error0;
  y_error0=goal_pose_.pose.position.y-current_pose_.pose.position.y;

  //error integral
  y_error_int+=y_error0*delt_t;
  // Apply windup limit to limit the size of the integral term
  if ( y_error_int > XY_ERROR_INT_LIMIT)
    y_error_int = XY_ERROR_INT_LIMIT;
  if ( y_error_int < -XY_ERROR_INT_LIMIT)
    y_error_int = -XY_ERROR_INT_LIMIT;

  velocity_ctrl_.twist.linear.y=Kp_pos*y_error0+Kd_pos*(y_error0-y_error1)/delt_t+Ki_pos*y_error_int;

  //limit   y   velocity_ctrl_
  if(velocity_ctrl_.twist.linear.y>XY_V_CTRL_LIMIT)
     velocity_ctrl_.twist.linear.y=XY_V_CTRL_LIMIT;
  if(velocity_ctrl_.twist.linear.y<-XY_V_CTRL_LIMIT)
     velocity_ctrl_.twist.linear.y=-XY_V_CTRL_LIMIT;

  /*  z  */

  z_error1=z_error0;
  z_error0=goal_pose_.pose.position.z-current_pose_.pose.position.z;

  //error integral
  z_error_int+=z_error0*delt_t;
  // Apply windup limit to limit the size of the integral term
  if ( z_error_int > Z_ERROR_INT_LIMIT)
    z_error_int = Z_ERROR_INT_LIMIT;
  if ( z_error_int < -Z_ERROR_INT_LIMIT)
    z_error_int = -Z_ERROR_INT_LIMIT;

  velocity_ctrl_.twist.linear.z=Kp_height*z_error0+Kd_height*(z_error0-z_error1)/delt_t+Ki_height*z_error_int;

  //limit   z   velocity_ctrl_
  if(velocity_ctrl_.twist.linear.z  > Z_V_CTRL_LIMIT)
     velocity_ctrl_.twist.linear.z  = Z_V_CTRL_LIMIT;
  if(velocity_ctrl_.twist.linear.z  < -Z_V_CTRL_LIMIT)
     velocity_ctrl_.twist.linear.z  = -Z_V_CTRL_LIMIT;


  /*  yaw  */

  yaw_error1=yaw_error0;
  yaw_error0=(tf::getYaw(goal_pose_.pose.orientation)-tf::getYaw(current_pose_.pose.orientation))*180/M_PI;
  //error integral
  yaw_error_int+=yaw_error0*delt_t;
  // Apply windup limit to limit the size of the integral term
  if ( yaw_error_int > YAW_ERROR_INT_LIMIT)
    yaw_error_int = YAW_ERROR_INT_LIMIT;
  if ( yaw_error_int < -YAW_ERROR_INT_LIMIT)
    yaw_error_int = -YAW_ERROR_INT_LIMIT;

  velocity_ctrl_.twist.angular.z=Kp_yaw*yaw_error0+Kd_yaw*(yaw_error0-yaw_error1)/delt_t+Ki_yaw*yaw_error_int;

  //limit   yaw   velocity_ctrl_
  if(velocity_ctrl_.twist.angular.z  > YAW_V_CTRL_LIMIT)
     velocity_ctrl_.twist.angular.z  = YAW_V_CTRL_LIMIT;
  if(velocity_ctrl_.twist.angular.z  < -YAW_V_CTRL_LIMIT)
     velocity_ctrl_.twist.angular.z  = -YAW_V_CTRL_LIMIT;


  velocity_ctrl_.header.frame_id  =current_pose_.header.frame_id;
  velocity_ctrl_.header.stamp     =current_pose_.header.stamp;
  outfile <<goal_pose_.pose.position.x<<" "<<goal_pose_.pose.position.y<<" "<<goal_pose_.pose.position.z<<" "\
         <<current_pose_.pose.position.x<<" "<<current_pose_.pose.position.y<<" "<<current_pose_.pose.position.z<<" "\
         <<velocity_ctrl_.twist.linear.x<<" "<<velocity_ctrl_.twist.linear.y<<" "<<velocity_ctrl_.twist.linear.z<<" "\
         <<Kp_pos*x_error0<<" "<<Kd_pos*(x_error0-x_error1)/delt_t<<" "<<Ki_pos*x_error_int<<" "\
         <<Kp_pos*y_error0<<" "<<Kd_pos*(y_error0-y_error1)/delt_t<<" "<<Ki_pos*y_error_int<<" "\
         <<Kp_height*z_error0<<" "<<Kd_height*(z_error0-z_error1)/delt_t<<" "<<Ki_height*z_error_int<<" "\
         <<std::endl;

}
inline void velocity_control_NEU(const geometry_msgs::TwistStamped& velocity_ctrl_,DJIDrone* drone){
  //drone->velocity_control(1,\
                          velocity_ctrl_.twist.linear.x,\
                          velocity_ctrl_.twist.linear.y,\
                          -velocity_ctrl_.twist.linear.z,\
                          velocity_ctrl_.twist.angular.z);
  drone->velocity_control(1,\
                          velocity_ctrl_.twist.linear.x,\
                          velocity_ctrl_.twist.linear.y,\
                          -velocity_ctrl_.twist.linear.z,\
                          0);
  // NEU FRAME=1(南X 西Y 上Z)
}
inline void velocity_control_BODY(const geometry_msgs::TwistStamped& velocity_ctrl_,DJIDrone* drone){
  drone->velocity_control(0,\
                          velocity_ctrl_.twist.linear.x,\
                          velocity_ctrl_.twist.linear.y,\
                          -velocity_ctrl_.twist.linear.z,\
                          0);
                         //velocity_ctrl_.twist.angular.z);
  //BODY FRAME=0 前x右y下z
}
void set_origin_position(){
  ros::spinOnce();
  origin_pose.header.frame_id 	=guidance_pose.header.frame_id;
  origin_pose.header.stamp      =guidance_pose.header.stamp;
  origin_pose.pose.position     =guidance_pose.pose.position;
  origin_pose.pose.orientation=tf::createQuaternionMsgFromYaw(0);

}

void set_goal_pose(int& goal_select)
{
  goal_select=goal_select % 3;
  goal_pose.header.frame_id="guidance";
  goal_pose.header.stamp=ros::Time::now();
  goal_pose.pose.position.x =origin_pose.pose.position.x+goal_x[goal_select];
  goal_pose.pose.position.y =origin_pose.pose.position.y+goal_y[goal_select];
  goal_pose.pose.position.z =origin_pose.pose.position.z+goal_z[goal_select];
  goal_pose.pose.orientation =tf::createQuaternionMsgFromYaw(goal_yaw[goal_select]*M_PI/180);
}


void fly_to_goal(DJIDrone* drone)
{
  // 20 Hz in while(ros::ok())
  //  rate_20.sleep();//
  ros::spinOnce();//
  if(guidancepose_stamp==lastpose_stamp)
  {
    velocity_ctrl.twist.linear.x=0;
    velocity_ctrl.twist.linear.y=0;
    velocity_ctrl.twist.linear.z=0;
    velocity_ctrl.twist.angular.z=0;
    ROS_INFO("No new Pose Feedback");
  }
  else
  {
    pidcontrol_pose(goal_pose,guidance_pose,velocity_ctrl);
    //ROS_INFO("guidance_pose x=%f,y=%f, z=%f,yaw=%f \nvelocity_ctrl x=%f,y=%f, z=%f,yaw=%f",\
               guidance_pose.pose.position.x,guidance_pose.pose.position.y,guidance_pose.pose.position.z,tf::getYaw(guidance_pose.pose.orientation)*180/M_PI,\
               velocity_ctrl.twist.linear.x,velocity_ctrl.twist.linear.y,velocity_ctrl.twist.linear.z,velocity_ctrl.twist.angular.z);
    velocity_control_NEU(velocity_ctrl,drone);
  }
}

void fly_to_origin(DJIDrone* drone)
{
  // 20 Hz in while(ros::ok())
  //  rate_20.sleep();//
  ros::spinOnce();//
  if(guidancepose_stamp==lastpose_stamp)
  {
    velocity_ctrl.twist.linear.x=0;
    velocity_ctrl.twist.linear.y=0;
    velocity_ctrl.twist.linear.z=0;
    velocity_ctrl.twist.angular.z=0;
    ROS_INFO("No new Pose Feedback");
  }
  else
  {
    pidcontrol_pose(origin_pose,guidance_pose,velocity_ctrl);
    //ROS_INFO("guidance_pose x=%f,y=%f, z=%f,yaw=%f \nvelocity_ctrl x=%f,y=%f, z=%f,yaw=%f",\
               guidance_pose.pose.position.x,guidance_pose.pose.position.y,guidance_pose.pose.position.z,tf::getYaw(guidance_pose.pose.orientation)*180/M_PI,\
               velocity_ctrl.twist.linear.x,velocity_ctrl.twist.linear.y,velocity_ctrl.twist.linear.z,velocity_ctrl.twist.angular.z);
    velocity_control_NEU(velocity_ctrl,drone);
  }
}
void hover_tag(DJIDrone* drone){// 20 Hz in while(ros::ok())

 // rate_20.sleep();//
  ros::spinOnce();//
  if(tagpose_stamp==lastpose_stamp)
  {
    velocity_ctrl.twist.linear.x=0;
    velocity_ctrl.twist.linear.y=0;
    velocity_ctrl.twist.linear.z=0;
    velocity_ctrl.twist.angular.z=0;
    ROS_INFO("No new Pose Feedback");
  }
  else

    {
      pidcontrol_pose(zero_pose,tag_pose,velocity_ctrl);
      ROS_INFO("tag_pose x=%f,y=%f, z=%f  \nvelocity_ctrl x=%f,y=%f, z=%f,yaw=%f",\
               tag_pose.pose.position.x,tag_pose.pose.position.y,tag_pose.pose.position.z,\
                velocity_ctrl.twist.linear.x,velocity_ctrl.twist.linear.y,velocity_ctrl.twist.linear.z,velocity_ctrl.twist.angular.z);
      velocity_control_BODY(velocity_ctrl,drone);
    }
}

void hover_apron(DJIDrone* drone){// 20 Hz in while(ros::ok())

 // rate_20.sleep();//
  ros::spinOnce();//
  if(apronpose_stamp==lastpose_stamp){
    velocity_ctrl.twist.linear.x=0;
    velocity_ctrl.twist.linear.y=0;
    velocity_ctrl.twist.linear.z=0;
    velocity_ctrl.twist.angular.z=0;
    ROS_INFO("No new Pose Feedback");
  }

  else
    {
      pidcontrol_pose(zero_pose,apron_pose,velocity_ctrl);
      //ROS_INFO("apron_pose x=%f,y=%f, z=%f  \nvelocity_ctrl x=%f,y=%f, z=%f,yaw=%f",\
               apron_pose.pose.position.x,apron_pose.pose.position.y,apron_pose.pose.position.z,\
                velocity_ctrl.twist.linear.x,velocity_ctrl.twist.linear.y,velocity_ctrl.twist.linear.z,velocity_ctrl.twist.angular.z);
      velocity_control_BODY(velocity_ctrl,drone);
    }
}


void take_off(DJIDrone* drone){
  drone->request_sdk_permission_control();//always return 1;
  ROS_INFO("Request Permission successed!");
  drone->takeoff();
  ROS_INFO("Taking off!");
  //此处加一个起飞成功的判断.
  //ros::Duration(3).sleep();
}
void landing(DJIDrone* drone){
  drone->landing();
  ROS_INFO("Landing!");
  //此处加一个降落成功的判断move_z_1.
}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "gdc_pos_node");
  ros::NodeHandle n;
  ros::NodeHandle pn("~");
  ros::Rate rate_20(20);

  pn.param<float>("kp_pos", Kp_pos, 1.2);
  pn.param<float>("ki_pos", Ki_pos, 0.1);
  pn.param<float>("kd_pos", Kd_pos, 0.3);
  pn.param<float>("kp_height", Kp_height, 0.9);
  pn.param<float>("ki_height", Ki_height, 0.08);
  pn.param<float>("kd_height", Kd_height, 0.2);
  pn.param<float>("kp_yaw", Kp_yaw, 1);
  pn.param<float>("ki_yaw", Ki_yaw, 0.08);
  pn.param<float>("kd_yaw", Kd_yaw, 0.3);
  pn.param<float>("XY_ERROR_INT_LIMIT", XY_ERROR_INT_LIMIT, 10);
  pn.param<float>("XY_V_CTRL_LIMIT", XY_V_CTRL_LIMIT, 10);
  pn.param<float>("Z_ERROR_INT_LIMIT", Z_ERROR_INT_LIMIT, 10);
  pn.param<float>("Z_V_CTRL_LIMIT", Z_V_CTRL_LIMIT, 10);
  pn.param<float>("YAW_ERROR_INT_LIMIT", YAW_ERROR_INT_LIMIT, 90);
  pn.param<float>("YAW_V_CTRL_LIMIT", YAW_V_CTRL_LIMIT, 90);
  pn.param<float>("goal1_x",goal_x[0],0);
  pn.param<float>("goal1_y",goal_y[0],0);
  pn.param<float>("goal1_z",goal_z[0],0);
  pn.param<float>("goal1_yaw",goal_yaw[0],0);

  pn.param<float>("goal2_x",goal_x[1],0);
  pn.param<float>("goal2_y",goal_y[1],0);
  pn.param<float>("goal2_z",goal_z[1],0);
  pn.param<float>("goal2_yaw",goal_yaw[1],0);

  pn.param<float>("goal3_x",goal_x[2],0);
  pn.param<float>("goal3_y",goal_y[2],0);
  pn.param<float>("goal3_z",goal_z[2],0);
  pn.param<float>("goal3_yaw",goal_yaw[2],0);

  pn.param<float>("apron_hover_height",apron_hover_height,1);

  ros::Subscriber goal_pose_sub    = n.subscribe("/uav_set_position",1,goal_pose_cb);
  ros::Subscriber guidance_pose_sub = n.subscribe("/guidance/pose", 1, guidance_pose_cb);
  ros::Subscriber tag_pose_sub  = n.subscribe("/tag_pose", 1, tag_pose_cb);
  ros::Subscriber apron_pose_sub  = n.subscribe("/apron_pose", 1, apron_pose_cb);
  ros::Subscriber mission_msg_sub  = n.subscribe("/uav_mission_msg",5, uav_mission_msg_cb);
  ros::Publisher uav_state_pub_    =n.advertise<std_msgs::UInt8  >("/uav_state",1);
  /*init zero_pose to hover uav*/
  zero_pose.header.frame_id="body_frame";
  zero_pose.header.stamp=ros::Time::now();
  zero_pose.pose.position.x=-0.25;
  zero_pose.pose.position.y=0;
  zero_pose.pose.position.z=-1.5;
  zero_pose.pose.orientation=tf::createQuaternionMsgFromYaw(0*M_PI/180);


  DJIDrone* drone = new DJIDrone(n);
  /*  request control ability  */
  drone->request_sdk_permission_control();//always return 1;
  //ROS_INFO("Request Permission successed!");

  outfile.open(log_dir);
  // set_origin_position();
  // origin_pose.pose.position.z=origin_pose.pose.position.z-0.8;
 // ROS_INFO("SET ORIGIN");
  //set_goal_pose(goal_select);

bool TAKINGOFF1=false;
bool TAKINGOFF2=false;

bool HOVER_APRON_1=false;
bool HOVERING_TAG=false;
bool FLYING_GOAL=false;
bool LANDIED=false;
bool FLYING_HOME=false;
int goal_select(0);
current_mission.MISSION=current_mission.UAV_STANDINGBY;
last_mission.MISSION=last_mission.UAV_STANDINGBY;
ROS_INFO("STANDING BY");
  while(ros::ok())
  {

    rate_20.sleep();
    ros::spinOnce();
    switch(current_mission.MISSION){
      case current_mission.UAV_TAKEOFF:
            ROS_INFO("REC TAKEOFF MISSION");
            if(!TAKINGOFF1){
              take_off(drone);
              take_off_stamp=ros::Time::now();
              TAKINGOFF1=true;
              ROS_INFO("TAKING OFF");
            }
            if(ros::Time::now()-take_off_stamp>ros::Duration(5))
            {
              if(!TAKINGOFF2){
                set_origin_position();
                origin_pose.pose.position.z=origin_pose.pose.position.z-1;
                pidcontrol_reinit();
                TAKINGOFF2=true;
                ROS_INFO("TAKING OFF");
              }
              fly_to_origin(drone);
            }
            //current_state=busy;
            break;

      case current_mission.UAV_FLYTOGOAL:
          if(!FLYING_GOAL)
          {
            set_goal_pose(goal_select);
            goal_select++;
            pidcontrol_reinit();
            FLYING_GOAL=true;
          }
          fly_to_goal(drone);
          break;
      case current_mission.UAV_FLYTOHOME:
          if(!FLYING_HOME)
          {
            origin_pose.pose.position.z=origin_pose.pose.position.z-1;
            pidcontrol_reinit();
            FLYING_HOME=true;
	    HOVER_APRON_1=false;
          }
          fly_to_origin(drone);
          break;
      case current_mission.UAV_HOVER_APRON:
            if(!HOVER_APRON_1)
            {
              pidcontrol_reinit();
              zero_pose.pose.position.z=-apron_hover_height;
              HOVER_APRON_1=true;
            }
            hover_apron(drone);
            break;
      case current_mission.UAV_SET_ORINGIN:
            hover_apron(drone);
            set_origin_position();
            ROS_INFO("SET ORINGIN SUCCESS !");
            break;

      case current_mission.UAV_LANDING:
            ROS_INFO("REC LANDING MISSION");
            if(!LANDIED)
            {
              landing(drone);
              LANDIED=true;
              ROS_INFO("LANDING");
            }
            break;
      case current_mission.UAV_HOVER_TAG:
            if(!HOVERING_TAG)
            {
              pidcontrol_reinit();
              HOVERING_TAG=true;
            }
            hover_tag(drone);break;
      case current_mission.UAV_STANDINGBY:
            TAKINGOFF1=false;
            TAKINGOFF2=false;
            HOVER_APRON_1=false;
            HOVERING_TAG=false;
            FLYING_GOAL=false;
            LANDIED=false;
            FLYING_HOME=false;
            break;
      default:
            break;
    }
    uav_state_pub_.publish(current_state);

  }

 // outfile.close();
  ros::spin();
  return 0;
}
