##Appendix

**Note: The ground station related msgs and srvs are not listed here, what named waypoint are not SDK waypoint protocol but an actionLib implemented logic.**

####Topic list:

```xml
/dji_sdk/acceleration
	info: acceleration data
	type: dji_sdk/Acceleration

/dji_sdk/activation
	info: acitvation result
	type: std_msgs/UInt8

/dji_sdk/attitude_quaternion
	info: quaternion data
	type: dji_sdk/AttitudeQuaternion

/dji_sdk/compass
	info: compass data
	type: dji_sdk/Compass

/dji_sdk/data_received_from_remote_device
	info: read transparent transmission data from mobile SDK
	type: dji_sdk/TransparentTransmissionData

/dji_sdk/flight_control_info
	info: drone control device info
	type: dji_sdk/FlightControlInfo

/dji_sdk/flight_status
	info: current flight status (please check the document for detailed meaning)
	type: std_msgs/UInt8

/dji_sdk/gimbal
	info: gimbal roll/yaw/pitch angle
	type: dji_sdk/Gimbal

/dji_sdk/global_position
	info: current GPS coordinate 
	type: dji_sdk/GlobalPosition

/dji_sdk/local_position
	info: current local position((X,Y,Z) coordinate, origin is the first recorded GPS point when server node start)
	type: dji_sdk/LocalPosition

/dji_sdk/odometry
	info: current odometry data
	type: nav_msgs/Odometry

/dji_sdk/power_status
	info: current battery info
	type: dji_sdk/PowerStatus #an uint8

/dji_sdk/rc_channels
	info: current channel values of RC controller
	type: dji_sdk/RCChannels

/dji_sdk/time_stamp
	info: current time stamp information
	type: dji_sdk/TimeStamp

/dji_sdk/velocity
	info: current velocity
	type: dji_sdk/Velocity

Note: the topics ending with `cancel`,`feedback`,`goal`,`result`,`status` are auto generated topics by the actionLib.

```

####Msg List

**Note: there are two timestamps in msgs. The first one is the timestamp in ROS header, which is the timestamp from ROS. The other one called ts comes from the drone, it is a timestamp from Matrice 100, unit in 1/400s.**

```xml
dji_sdk/GlobalPositionNavigationAction
dji_sdk/GlobalPositionNavigationActionFeedback
dji_sdk/GlobalPositionNavigationActionGoal
dji_sdk/GlobalPositionNavigationActionResult
dji_sdk/GlobalPositionNavigationFeedback
dji_sdk/GlobalPositionNavigationGoal
dji_sdk/GlobalPositionNavigationResult
	auto generated by global navigation action

dji_sdk/LocalPositionNavigationAction
dji_sdk/LocalPositionNavigationActionFeedback
dji_sdk/LocalPositionNavigationActionGoal
dji_sdk/LocalPositionNavigationActionResult
dji_sdk/LocalPositionNavigationFeedback
dji_sdk/LocalPositionNavigationGoal
dji_sdk/LocalPositionNavigationResult
	auto generated by local navigation action

dji_sdk/DroneTaskAction
dji_sdk/DroneTaskActionFeedback
dji_sdk/DroneTaskActionGoal
dji_sdk/DroneTaskActionResult
dji_sdk/DroneTaskFeedback
dji_sdk/DroneTaskGoal
dji_sdk/DroneTaskResult
	auto generated by drone task action

dji_sdk/WaypointNavigationAction
dji_sdk/WaypointNavigationActionFeedback
dji_sdk/WaypointNavigationActionGoal
dji_sdk/WaypointNavigationActionResult
dji_sdk/WaypointNavigationFeedback
dji_sdk/WaypointNavigationGoal
dji_sdk/WaypointNavigationResult
	auto generate by waypoint navigation action

dji_sdk/Acceleration
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	float32 ax
	float32 ay
	float32 az

dji_sdk/AttitudeQuaternion
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	float32 q0
	float32 q1
	float32 q2
	float32 q3
	float32 wx
	float32 wy
	float32 wz

dji_sdk/Compass
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	int8 x
	int8 y
	int8 z

dji_sdk/FlightControlInfo
	uint8 control_mode
	uint8 cur_ctrl_dev_in_navi_mode #0:rc, 1:mobile, 2:onboard
	uint8 serial_req_status #0:disabled, 1:enabled
	uint8 virtual_rc_status #0:disabled, 1:enabled

dji_sdk/Gimbal
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	float32 pitch
	float32 yaw
	float32 roll

dji_sdk/GlobalPosition
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	#latitude is in angle
	float64 latitude
	#longitude is in angle
	float64 longitude
	float32 altitude
	float32 height
	#reliablity [0,5]
	int8 health

dji_sdk/LocalPosition
	#    North(x)
	#   /
	#  /
	# /______East(y)
	# |
	# |
	# Donw (-z)
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	float32 x
	float32 y
	float32 z

dji_sdk/RCChannels
	# RC Map
	#
	#  mode:
	#
	# +8000 <--->  0  <---> -8000
	#  API  <---> ATT <--->  POS
	#
	# CH3(throttle) +10000            CH1(pitch) +10000
	#             ^                            ^
	#             |                            |                  / -10000
	#  CH2(yaw)   |                CH0(roll)   |                 /
	# -10000 <-----------> +10000 -10000 <-----------> +10000   H(gear_up)
	#             |                            |                 \
	#             |                            |                  \ -4545
	#             V                            V
	#          -10000                       -10000
	# The range in RCChannles is different from the range of Virtual RC command.

	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	float32 roll
	float32 pitch
	float32 yaw
	float32 throttle
	float32 mode
	float32 gear_up

dji_sdk/TimeStamp
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	uint32 time 	#unit is 1/400s
	uint32 time_ns  #unit is nano second, overflow every 4 sec
	uint8 sync_flag

dji_sdk/TransparentTransmissionData
	uint8[] data

dji_sdk/Velocity
	std_msgs/Header header
	  uint32 seq
	  time stamp
	  string frame_id
	int32 ts
	float32 vx
	float32 vy
	float32 vz
	uint8 health_flag (0:not trustable or 1:trustable)

dji_sdk/Waypoint
	#latitude is in degree
	float64 latitude
	#longitude is in degree
	float64 longitude
	float32 altitude
	#heading is in [-180,180]
	int16 heading
	#stay time is in second
	uint16 staytime

dji_sdk/WaypointList
	dji_sdk/Waypoint[] waypoint_list


```

####Service List
```xml
/dji_sdk/attitude_control
	info: drone attitude control service
	type: dji_sdk/AttitudeControl

/dji_sdk/camera_action_control
	info: camera action control service
	type: dji_sdk/CameraActionControl

/dji_sdk/drone_arm_control
	info: arm/disarm drone service
	type: dji_sdk/DroneArmControl

/dji_sdk/drone_task_control
	info: drone task control service
	type: dji_sdk/DroneTaskAction

/dji_sdk/gimbal_angle_control
	info: gimbal angle control service
	type: dji_sdk/GimbalAngleService

/dji_sdk/gimbal_speed_control
	info: gimbal angular speed control service
	type: dji_sdk/GimbalSpeedControl

#note: for navigation purpose, user should keep sending the position offset 
/dji_sdk/global_position_control
	info: global position control service
	type: dji_sdk/GlobalPositionControl

#note: for navigation purpose, user should keep sending the position offset 
/dji_sdk/local_position_control
	info: local position control service
	type: dji_sdk/LocalPositionControl

/dji_sdk/send_data_to_remote_device
	info: send transparent transmission data service
	type: dji_sdk/SendDataToRemoteDevice

/dji_sdk/sdk_permission_control
	info: obtain or release control abilitiy service
	type: dji_sdk/SDKPermissionControl

/dji_sdk/velocity_control
	info: velocity control service
	type: dji_sdk/VelocityControl

/dji_sdk/version_check
	info: check sdk version service
	type: dji_sdk/VersionCheck

/dji_sdk/virtual_rc_enable_control
	info: enable/disable Virtual RC service
	type: dji_sdk/VirtualRCEnableControl

/dji_sdk/virtual_rc_data_control
	info: send Virtual RC data service
	type: dji_sdk/VirtualRCDataControl

```

####Srv List
```xml
dji_sdk/AttitudeControl
	uint8 flag
	float32 x
	float32 y
	float32 z
	float32 yaw
	---
	bool result 

dji_sdk/CameraActionControl
	#takePicture: 0
	#startRecord: 1
	#stopRecord:  2
	uint8 camera_action
	---
	bool result

dji_sdk/DroneArmControl
	#arm: 1
	#disarm: 0
	uint8 arm
	---
	bool result

dji_sdk/DroneTaskControl
	#takeoff: 4
	#landing: 6
	#gohome:  1
	uint8 task
	---
	bool result

dji_sdk/GimbalAngleControl
	int16 yaw
	int16 roll
	int16 pitch
	int16 duration
	bool absolute_or_incremental	#absolute: 1
	bool roll_cmd_ignore
	bool pitch_cmd_ignore
	bool yaw_cmd_ignore
	---
	bool result 

dji_sdk/GimbalSpeedControl
	int16 yaw_rate
	int16 roll_rate
	int16 pitch_rate
	---
	bool result 

dji_sdk/GlobalPositionControl
	float64 latitude
	float64 longitude
	float32 altitude
	float32 yaw
	---
	bool result

dji_sdk/LocalPositionControl
	float32 x
	float32 y
	float32 z
	float32 yaw
	---
	bool result

dji_sdk/SDKPermissionControl
	#requestControl: 1
	#releaseControl: 0
	uint8 control_enable
	---
	bool result

dji_sdk/SendDataToRemoteDevice
	uint8[] data #100 byte at most
	---
	bool result

dji_sdk/VelocityControl
	# body frame: 0, with stable mode 
	#world frame: 1, with stable mode
	uint8 frame
	float32 vx
	float32 vy
	float32 vz
	float32 yawRate
	---
	bool result

dji_sdk/VersionCheck
	---
	bool result

dji_sdk/VirtualRCEnable
	uint8 enable
	uint8 if_back_to_real
	#switch back to real RC when 1
	#behave like RC signal lost when 0
	---
	bool result

dji_sdk/VirtualRCDataControl
	#IMPORTANT: the channel range in VRC is different
	#			from the channel range in broadcast data.
	#
	# VRC channel range is [1024-660, 1024+660]
	# For Matrice 100: 
	#		channle_data[0]: Roll
	#		channle_data[1]: pitch 
	#		channle_data[2]: throttle 
	#		channle_data[3]: yaw
	#		channle_data[4]: gear (1684:up, 1324:down)
	#		channle_data[6]: mode (1552:P, 1024:A, 496:F)
	uint32[16] channel_data
	---
	bool result

```

####Action Server List
```xml

dji_sdk/drone_task_action
	info: same to drone action control service
	type: dji_sdk/DroneTask.action

dji_sdk/global_position_navigation_action
	info: global navigation action
	type: dji_sdk/GlobalPositionNavigation.action

dji_sdk/local_position_navigation_action
	info: local navigation action
	type: dji_sdk/LocalPositionNavigation.action

dji_sdk/waypoint_navigation_action
	info: waypoint navigation action
	type: dji_sdk/WaypointNavigation.action
```

####Action File List
```xml

DroneTask.action
	uint8 task 
	---
	bool result
	---
	uint8 progress 

GlobalPositionNavigation.action
	#latitude is in degree
	float64 latitude
	#longitude is in degree
	float64 longitude
	float32 altitude
	---
	bool result
	---
	#progress is in percent
	uint8 latitude_progress
	uint8 longitude_progress
	uint8 altitude_progress


LocalPositionNavigation.action
	#x,y,z are in meters
	float32 x
	float32 y
	float32 z
	---
	bool result
	---
	#progress is in percent
	uint8 x_progress 
	uint8 y_progress 
	uint8 z_progress 

WaypointNavigation.action
	WaypointList waypoint_list
	---
	bool result
	---
	#progress is in percent
	uint8 latitude_progress
	uint8 longitude_progress
	uint8 altitude_progress
	uint8 index_progress

