#include "cv_bgr/cv_bgr.h"
#include <math.h>

//#define Debug_BGR

using namespace std;
using namespace cv;
static const std::string INPUT = "Input";
static const std::string OUTPUT = "Output";
float last_angel;


CV_BGR::CV_BGR(ros::NodeHandle& nh_,ros::NodeHandle& pnh):it_(nh_)
{
    if(!pnh.getParam("function_a", function_a)){
    ROS_WARN("No function_a tags specified");
  }
    if(!pnh.getParam("function_b", function_b)){
    ROS_WARN("No function_b tags specified");
  }
    image_sub_ = it_.subscribe("/image_raw",
                              1, &CV_BGR::convert_callback, this);
    image_pub_ = it_.advertise("/image_converter/output_video", 1);

    //tagcenter_pub_ = nh_.advertise<cv_bgr::myCenter>("tag_center_3d",1);
    tag_pose_pub_ = nh_.advertise<geometry_msgs::PoseStamped>("tag_pose",1);
    pnh.param<bool>("show_img", show_img, false);

    pnh.param<int>("width", width, 640);
    pnh.param<int>("height", height, 480);
    pnh.param<int>("iLowH", iLowH, 141);
    pnh.param<int>("iHighH", iHighH, 180);
    pnh.param<int>("iLowS", iLowS, 87);
    pnh.param<int>("iHighS", iHighS, 255);
    pnh.param<int>("iLowV", iLowV, 60);
    pnh.param<int>("iHighV", iHighV, 255);
    pnh.param<int>("thresh", thresh, 35);  

    pnh.param<double>("fx", fx, 489.355407);
    pnh.param<double>("fy", fy, 490.524912);
    pnh.param<double>("cx", cx, 316.109536);
    pnh.param<double>("cy", cy, 247.635580);


 
    img_ball.x = 0;
    img_ball.y = 0;
    img_ball.r = 0.0;

    ball.x = 0;
    ball.y = 0;
    ball.d = 0.0;
    ball.angle = 0.0;
    getRect = false;
    ballRect = Rect(0,0,0,0);
#ifdef Debug_BGR
    cv::namedWindow(INPUT);
    cv::namedWindow(OUTPUT);
#endif
}

CV_BGR::~CV_BGR()
{
#ifdef Debug_BGR
    cv::namedWindow(INPUT);
    cv::namedWindow(OUTPUT);
#endif
}
double CV_BGR::tic() {
    struct timeval t;
    gettimeofday(&t, NULL);
    return ((double)t.tv_sec + ((double)t.tv_usec)/1000000.);
}
void CV_BGR::convert_callback(const sensor_msgs::ImageConstPtr& msg)
{
    cv_bridge::CvImagePtr cv_ptr; // 声明一个CvImage指针的实例

    try
    {
        cv_ptr =  cv_bridge::toCvCopy(msg, sensor_msgs::image_encodings::BGR8); //将ROS消息中的图象信息提取，生成新cv类型的图象，复制给CvImage指针
    }
    catch(cv_bridge::Exception& e)  //异常处理
    {
        ROS_ERROR("cv_bridge exception: %s", e.what());
        return;
    }
    double t0 = tic();
    ballRect = get_rect(cv_ptr->image);
    if(getRect)
    {
    	find_ball(cv_ptr->image,ballRect);
      if(ball.d <= 1.7)
      {
        get_angle(cv_ptr->image,ballRect);
        if(ball.angle-last_angel>=15||ball.angle-last_angel<=15)
          ball.angle=0;
      }
      else
        ball.angle=0;
    }
     //tagcenter_pub_.publish(ball);
     last_angel=ball.angle;
     tag_pose.pose.position.x=ball.x;
     tag_pose.pose.position.y=ball.y;
     tag_pose.pose.position.z=ball.d;
     tag_pose.pose.orientation=tf::createQuaternionMsgFromYaw(ball.angle*M_PI/180);
     tag_pose.header.frame_id="camera_down";
     tag_pose.header.stamp=ros::Time::now();

     tag_pose_pub_.publish(tag_pose);



    ROS_INFO ("cost time = [%f]",tic()-t0);
    if(show_img)
    image_show(cv_ptr->image); //得到了cv::Mat类型的图象，在CvImage指针的image中，将结果传送给处理函数
    //image_pub_.publish(cv_ptr->toImageMsg());

}

void CV_BGR::image_show(cv::Mat& img)
{
   //cv::circle(img, cv::Point(50, 50), 10, CV_RGB(255,0,0));
   cv::Mat img_out;
   cv::cvtColor(img, img_out, CV_BGR2GRAY);  //转换成灰度图象
   cv::resize(img_out,img_out,cv::Size(width,height));

   cv::imshow(INPUT, img);
   cv::imshow(OUTPUT, img_out);
   cv::moveWindow(INPUT, 0, 0);
   cv::waitKey(1);
}


cv::Rect CV_BGR::get_rect(cv::Mat src)
{
   vector<Mat> img ;
   pyrDown(src,src);
   split(src,img);
   Mat img_out = img[1] - img[2];
   threshold(img_out, img_out, thresh, 255, THRESH_BINARY); 
   Mat element = getStructuringElement(MORPH_ELLIPSE, Size(5, 5));

   morphologyEx(img_out, img_out, MORPH_OPEN, element);
   float area_ratio_limit = 0.65;
   double cont_area = 0.0;
   double circle_area = 0.0;
   double area_ratio = 0.0;
   double max_area_ratio = 0.0;
   int x = 0, y = 0;
   float r = 0.0;
   Rect rect(0,0,0,0);
   vector< vector<cv::Point> >  contours;
   vector<Vec4i> hierarchy;

   findContours(img_out, contours, hierarchy, RETR_TREE, CV_CHAIN_APPROX_NONE);

   vector<Point2f>  center(contours.size());
   vector<float>  radius(contours.size());

   getRect = false;

   for (unsigned int i = 0; i<contours.size(); i++)
   {


       //find circle
       minEnclosingCircle(contours[i], center[i], radius[i]);
       if(radius[i] < 3)
       {
           continue;
       }
       cont_area = contourArea(contours[i], false);
       circle_area = 3.14 * radius[i] * radius[i];
       area_ratio = cont_area / circle_area;

       if (area_ratio < area_ratio_limit)
       {
           continue;
       }
       if (cont_area < max_area_ratio)
       {
           continue;
       }
       max_area_ratio = cont_area;

       x = int(center[i].x);
       y = int(center[i].y);
       r = radius[i];
       getRect = true;

   }
    // tagcenter_pub_.publish(ball);
    if(getRect)
    {
	
        rect.x = 2*(x - r) > 0 ? 2*(x - r):0;
        rect.y = 2*(y - r) > 0 ? 2*(y - r):0;
        rect.width = (rect.x + 4*r) > 640 ? 640 - rect.x : 4*r;
        rect.height = (rect.y + 4*r) > 480 ? 480 - rect.y : 4*r;

    }
  // cout<<"x = "<<rect.x<<"y = "<<rect.y<<" w = "<<rect.width<<" h = "<<rect.height<<endl;
   return rect;

}


cv::Mat CV_BGR::find_ball(cv::Mat src, cv::Rect rect)
{
   //Mat img(rect.height - 1, rect.width - 1, CV_8UC3, Scalar(0,0,0));
   //cvtColor(src(rect), img, CV_BGR2HSV);

   vector<Mat> img ;
   split(src(rect),img);
   Mat img_out = img[1] - img[2];
   threshold(img_out, img_out, thresh, 255, THRESH_BINARY); 
   //imshow("find_ball_img",img_out);
  

   Mat element = getStructuringElement(MORPH_ELLIPSE, Size(7, 7));
   morphologyEx(img_out, img_out, MORPH_OPEN, element);

   float area_ratio_limit = 0.5;
   double cont_area = 0.0;
   double circle_area = 0.0;
   double area_ratio = 0.0;
   double max_area_ratio = 0.0;
   int x = 0, y = 0;
   float r = 0.0;
   vector< vector<cv::Point> >  contours;
   vector<Vec4i> hierarchy;

   findContours(img_out, contours, hierarchy, RETR_TREE, CV_CHAIN_APPROX_NONE);

   vector<Point2f>  center(contours.size());
   vector<float>  radius(contours.size());


   for (unsigned int i = 0; i<contours.size(); i++)
   {

       minEnclosingCircle(contours[i], center[i], radius[i]);
       if(radius[i]<5)
       {
           continue;
       }
       //cout << "radius = " << radius[i] << endl;
       //cout << "arcLength = " <<arcLength(contours[i], true)<<endl;
       cont_area = contourArea(contours[i], false);
       circle_area = 3.14 * radius[i] * radius[i];
       area_ratio = cont_area / circle_area;
       //cout << "cont_area = " << cont_area << endl;
       //cout << "circle_area = " << circle_area << endl;
       if (area_ratio < area_ratio_limit)
       {
           continue;
       }
       if (cont_area < max_area_ratio)
       {
           continue;
       }
       max_area_ratio = cont_area;

       x = int(center[i].x);
       y = int(center[i].y);
       r = radius[i];

   }

   if(r >= 8)
   {   
    img_ball.x = x;
    img_ball.y = y;
    img_ball.r = r;

       ball.x = x + rect.x;
       ball.y = y + rect.y;

       circle(src, Point(ball.x, ball.y), 2, Scalar(255),-1);
       circle(src, Point(ball.x, ball.y), r, Scalar(255),2);
//
       ball.d = function_a*pow(r,function_b)/100;
       ball.x = ball.d * (ball.x - cx) / fx ;
       ball.y = ball.d * (ball.y - cy) / fy;
       ROS_INFO (" r = [%f]",r);
       ROS_INFO (" d = [%f]",ball.d);
      
   }


   return img_out;
}

bool CV_BGR::get_angle(cv::Mat src, cv::Rect rect)
{
   vector<Mat> img ;
   split(src(rect),img);
   Mat img_out = img[1] - img[2];
   threshold(img_out, img_out, thresh, 255, THRESH_BINARY); 
   img_out = ~img_out;
   Mat element = getStructuringElement(MORPH_RECT, Size(5, 5));

   morphologyEx(img_out, img_out, MORPH_OPEN, element);


   //cvtColor(src(rect), img_out, CV_BGR2GRAY);

   vector< vector<Point> >  contours;
   vector<Vec4i> hierarchy;
   int thresh = 30;
   Canny(img_out, img_out, thresh, thresh*3, 3);

   findContours(img_out, contours, hierarchy, RETR_EXTERNAL, CV_CHAIN_APPROX_NONE);
   
   vector<Moments> mu(contours.size() );
    //计算轮廓矩
    for( int i = 0; i < contours.size(); i++ )
    { 
	mu[i] = moments( contours[i], false);
    }
    //计算轮廓的质心
    vector<Point2f> mc( contours.size() );
    for( int i = 0; i < contours.size(); i++ )
    { 
	mc[i] = Point2d( mu[i].m10/mu[i].m00 , mu[i].m01/mu[i].m00 ); 
    }
    float a,b,c;
    float angle = 0.0;
    bool getAngle = false;
    float max_area_ratio = 0.0;
    float x , y;
    ROS_INFO ("contours.size() = [%d]",contours.size());
    for( int i = 0; i< contours.size(); i++ )
    {
        //if((mc[i].x - rect.width/2 > rect.width/4)||(mc[i].y - rect.height/2 > rect.height/4))
	//{
	//continue;
	//}
       //ROS_INFO ("num[%d] , length = [%f]",i,arcLength( contours[i], true ));
       if (arcLength( contours[i], true ) >= 2*(rect.width-2))
       {
           continue;
       }

       if (mu[i].m00 < max_area_ratio)
       {
           continue;
       }
       max_area_ratio = mu[i].m00;
	


	getAngle = true;
        x = mc[i].x;
	y = mc[i].y;
	
/*
        a = mu[i].m20/mu[i].m00 - pow(mc[i].x,2);
        b = 2*( mu[i].m11/mu[i].m00 - mc[i].x*mc[i].y);
	
        c = mu[i].m02/mu[i].m00 - pow(mc[i].y,2);
        angle = atan2(b,a-c)/2;
	printf(" num[%d] , length = [%f],mu30 =  %.4f  mu03 =  %.4f \n", 
		i,arcLength( contours[i], true ),mu[i].mu30,mu[i].mu03);
*/
    }

        //printf("a = %.4f - b = %.4f - c =  %.4f\n ", a, b, c);
	if(getAngle)
	{
	angle = atan2(y - img_ball.y,x - img_ball.x)*180/3.14;

	angle += 90;
	if(angle >= 180)
	{
	angle -= 360;
	}

        circle(src, Point(x + rect.x, y + rect.y), 2, Scalar(0,0,255),-1);
	//line(src,Point(img_ball.x + rect.x, img_ball.y + rect.y),Point(x + rect.x, y + rect.y),cvScalar(0,255,0), 3);
        char tam[100];
        sprintf(tam, " angle%.2f",angle);
        putText(src, tam, Point(x + rect.x, y + rect.y), FONT_HERSHEY_SIMPLEX, 0.5, cvScalar(0,220,120),1.5);
	
        printf("angle =  %.4f \n", angle);
        ball.angle = angle;
	}
	
    return getAngle;
}


