#include "find_apron/find_apron.h"



using namespace std;
using namespace cv;
static const std::string INPUT = "Input";
static const std::string OUTPUT = "Output";

FIND_APRON::FIND_APRON(ros::NodeHandle& nh_,ros::NodeHandle& pnh):it_(nh_)
{
    if(!pnh.getParam("function_a", function_a)){
      ROS_WARN("No function_a tags specified");
    }
    image_sub_ = it_.subscribe("/image_raw",
                              1, &FIND_APRON::convert_callback, this);
    //image_pub_ = it_.advertise("/image_converter/output_video2", 1);

    //tagcenter_pub_ = nh_.advertise<find_apron::apron_center>("tag_ring_center",1);
    apron_pose_pub_ = nh_.advertise<geometry_msgs::PoseStamped>("apron_pose",1);


    pnh.param<int>("width", width, 640);
    pnh.param<int>("height", height, 480);
    pnh.param<int>("iLowH", iLowH, 141);
    pnh.param<int>("iHighH", iHighH, 180);
    pnh.param<int>("iLowS", iLowS, 87);
    pnh.param<int>("iHighS", iHighS, 255);
    pnh.param<int>("iLowV", iLowV, 60);
    pnh.param<int>("iHighV", iHighV, 255);
    pnh.param<int>("thread", thread, 35);

    pnh.param<bool>("show_img", show_img, false);
    pnh.param<bool>("redApron", redApron, true);
    pnh.param<double>("function_a", function_a, 1594.0);
    pnh.param<double>("function_b", function_b, -0.9427);

    pnh.param<double>("fx", fx, 489.355407);
    pnh.param<double>("fy", fy, 490.524912);
    pnh.param<double>("cx", cx, 316.109536);
    pnh.param<double>("cy", cy, 247.635580);

    //pnh.getParam("iLowV", function_a);
    //pnh.getParam("iHighV", function_b);


    ring.x = 0;
    ring.y = 0;
    ring.d = 0.0;

    getRect = false;
    ringRect = Rect(0,0,0,0);

}

FIND_APRON::~FIND_APRON()
{

}
double FIND_APRON::tic() {
    struct timeval t;
    gettimeofday(&t, NULL);
    return ((double)t.tv_sec + ((double)t.tv_usec)/1000000.);
}


void FIND_APRON::convert_callback(const sensor_msgs::ImageConstPtr& msg)
{
    cv_bridge::CvImagePtr cv_ptr; // 声明一个CvImage指针的实例
    try
    {
        cv_ptr =  cv_bridge::toCvCopy(msg, sensor_msgs::image_encodings::BGR8); //将ROS消息中的图象信息提取，生成新cv类型的图象，复制给CvImage指针
    }
    catch(cv_bridge::Exception& e)  //异常处理
    {
        ROS_ERROR("cv_bridge exception: %s", e.what());
        return;
    }
    double t0 = tic();
    ringRect = get_rect(cv_ptr->image);
    if(getRect)
    {
        find_ring(cv_ptr->image,ringRect);
    }
    //tagcenter_pub_.publish(ring);

    apron_pose.pose.position.x=ring.x;
    apron_pose.pose.position.y=ring.y;
    apron_pose.pose.position.z=ring.d;
    apron_pose.pose.orientation=tf::createQuaternionMsgFromYaw(0.0);
    apron_pose.header.frame_id="camera_down";
    apron_pose.header.stamp=ros::Time::now();

    apron_pose_pub_.publish(apron_pose);

    ROS_INFO ("cost time = [%f]",tic()-t0);
    if(show_img)
    image_show(cv_ptr->image);

}

void FIND_APRON::image_show(cv::Mat& img)
{
   //cv::circle(img, cv::Point(50, 50), 10, CV_RGB(255,0,0));
   cv::Mat img_out;
   cv::cvtColor(img, img_out, CV_BGR2GRAY);  //转换成灰度图象
   cv::resize(img_out,img_out,cv::Size(width,height));

   cv::imshow(INPUT, img);
   cv::imshow(OUTPUT, img_out);
   cv::moveWindow(INPUT, 0, 0);
   cv::moveWindow(OUTPUT, 700, 0);
   cv::waitKey(1);
}

cv::Rect FIND_APRON::get_rect(cv::Mat src)
{
   pyrDown(src,src);
   vector<Mat> img;
   split(src,img); 
   Mat img_out;
   if(redApron)
   img_out = img[2] - img[0];
   else
   img_out = img[0] - img[2];

   threshold(img_out,img_out,thread,255,THRESH_BINARY);

   Mat element = getStructuringElement(MORPH_ELLIPSE, Size(5, 5));

   morphologyEx(img_out, img_out, MORPH_CLOSE, element);
   float area_ratio_limit = 0.65;
   double cont_area = 0.0;
   double circle_area = 0.0;
   double area_ratio = 0.0;
   double max_area_ratio = 0.0;
   int x = 0, y = 0;
   float r = 0.0;
   Rect rect(0,0,0,0);
   vector< vector<cv::Point> >  contours;
   vector<Vec4i> hierarchy;

   findContours(img_out, contours, hierarchy, RETR_TREE, CV_CHAIN_APPROX_NONE);

   vector<Point2f>  center(contours.size());
   vector<float>  radius(contours.size());

   getRect = false;

   for (unsigned int i = 0; i<contours.size(); i++)
   {


       //find circle
       minEnclosingCircle(contours[i], center[i], radius[i]);
       if(radius[i] < 20)
       {
           continue;
       }
       cont_area = contourArea(contours[i], false);
       circle_area = 3.14 * radius[i] * radius[i];
       area_ratio = cont_area / circle_area;

       if (area_ratio < area_ratio_limit)
       {
           continue;
       }
       if (cont_area < max_area_ratio)
       {
           continue;
       }
       max_area_ratio = cont_area;

       x = int(center[i].x);
       y = int(center[i].y);
       r = radius[i];
       getRect = true;

   }

    if(getRect)
    {
        rect.x = 2*(x - r) > 0 ? 2*(x - r):0;
        rect.y = 2*(y - r) > 0 ? 2*(y - r):0;
        rect.width = (rect.x + 4*r) > 640 ? 640 - rect.x : 4*r;
        rect.height = (rect.y + 4*r) > 480 ? 480 - rect.y : 4*r;
    }
  // cout<<"x = "<<rect.x<<"y = "<<rect.y<<" w = "<<rect.width<<" h = "<<rect.height<<endl;
   return rect;
}

cv::Mat FIND_APRON::find_ring(cv::Mat src, cv::Rect &rect)
{

   vector<Mat> img;
   split(src(rect),img); 
   Mat img_out;
   if(redApron)
   img_out = img[2] - img[0];
   else
   img_out = img[0] - img[2];

   threshold(img_out,img_out,thread,255,THRESH_BINARY);


   Mat element = getStructuringElement(MORPH_ELLIPSE, Size(7, 7));
   morphologyEx(img_out, img_out, MORPH_CLOSE, element);

   float area_ratio_limit = 0.5;
   double cont_area = 0.0;
   double circle_area = 0.0;
   double area_ratio = 0.0;
   double max_area_ratio = 0.0;
   int x = 0, y = 0;
   float r = 0.0;
   vector< vector<cv::Point> >  contours;
   vector<Vec4i> hierarchy;

   findContours(img_out, contours, hierarchy, RETR_TREE, CV_CHAIN_APPROX_NONE);

   vector<Point2f>  center(contours.size());
   vector<float>  radius(contours.size());


   for (unsigned int i = 0; i<contours.size(); i++)
   {

       minEnclosingCircle(contours[i], center[i], radius[i]);
       if(radius[i]<40)
       {
           continue;
       }
       //cout << "radius = " << radius[i] << endl;
       //cout << "arcLength = " <<arcLength(contours[i], true)<<endl;
       cont_area = contourArea(contours[i], false);
       circle_area = 3.14 * radius[i] * radius[i];
       area_ratio = cont_area / circle_area;
       //cout << "cont_area = " << cont_area << endl;
       //cout << "circle_area = " << circle_area << endl;
       if (area_ratio < area_ratio_limit)
       {
           continue;
       }
       if (cont_area < max_area_ratio)
       {
           continue;
       }
       max_area_ratio = cont_area;

       x = int(center[i].x);
       y = int(center[i].y);
       r = radius[i];

   }

   if(r >= 40)
   {

       ring.x = x + rect.x;
       ring.y = y + rect.y;

       circle(src, Point(ring.x, ring.y), 2, Scalar(255),-1);
       circle(src, Point(ring.x, ring.y), r, Scalar(255),2);
//
       ring.d = function_a*pow(r,function_b)/100;
       ring.x = ring.d * (ring.x - cx) / fx ;
       ring.y = ring.d * (ring.y - cy) / fy;
       ROS_INFO (" r = [%f]",r);
       ROS_INFO (" d = [%f]",ring.d);

   }

   return img_out;
}


